from .key_message import GeneralKeyMessage
from .key_value import GeneralKeyValue
from .name_value import GeneralNameValue
from .result_error import GeneralResultError
from .status import GeneralStatus
from .success import GeneralSuccess
from .type_value import GeneralTypeValue
from .validated import GeneralValidated

__all__ = [
    'GeneralKeyMessage',
    'GeneralKeyValue',
    'GeneralNameValue',
    'GeneralResultError',
    'GeneralStatus',
    'GeneralSuccess',
    'GeneralTypeValue',
    'GeneralValidated',
]



from moodle.attr import dataclass
from typing import Optional


@dataclass
class EnrolmentMethod:
    """Enrolment Method

    Args:
        id (int): id of course enrolment instance
        courseid (int): id of course
        type (str): type of enrolment plugin
        name (str): name of enrolment plugin
        status (str): status of enrolment plugin
        wsfunction (Optional[str]): webservice function to get more information
    """
    id: int
    courseid: int
    type: str
    name: str
    status: str
    wsfunction: Optional[str]



from typing import List

from moodle import BaseMoodle
from . import Urls, View


class BaseUrl(BaseMoodle):
    def get_urls_by_courses(self, courseids: List[int]) -> Urls:
        """Returns a list of urls in a provided list of courses, if no list is provided all urls that the user can view will be returned.

        Args:
            courseids (List[int]): Array of course ids

        Returns:
            Urls: List of Url
        """
        res = self.moodle.post(
            'mod_url_get_urls_by_courses',
            courseids=courseids,
        )
        return self._tr(Urls, **res)

    def view_url(self, urlid: int) -> View:
        """Trigger the course module viewed event and update the module completion status.

        Args:
            urlid (int): url instance id

        Returns:
            View: Response
        """
        res = self.moodle.post('mod_url_view_url', urlid=urlid)
        return self._tr(View, **res)



from typing import List

from moodle import MoodleWarning
from moodle.attr import dataclass, fields


@dataclass
class TokenPrivateToken:
    """Token and Private Token

    Args:
        token (str): A valid WebService token for the official mobile app service.
        privatetoken (str): Private token used for auto-login processes.
        warnings (List[MoodleWarning]): list of warnings
    """
    token: str
    privatetoken: str
    warnings: List[MoodleWarning] = fields(MoodleWarning)



from moodle.attr import dataclass


@dataclass
class FilepathFilename:
    """Files or directories to be deleted.

    Args:
        filepath (str): Path to the file or directory to delete.
        filename (str): Name of the file to delete.
    """
    filepath: str
    filename: str



from moodle.attr import dataclass
from typing import List


@dataclass
class Field:
    """Field

    Args:
        name (str): name
        shortname (str): shortname
        type (str): type
        editfieldurl (str): edit field url
        id (int): id
    """
    name: str
    shortname: str
    type: str
    editfieldurl: str
    id: int


@dataclass
class Category:
    """Category

    Args:
        id (int): id
        nameeditable (str): inplace editable name
        addfieldmenu (str): addfieldmenu
        fields (List[Field]): list of Field
    """
    id: int
    nameeditable: str
    addfieldmenu: str
    fields: List[Field]


@dataclass
class ReloadTemplate:
    """Reloaded Template

    Args:
        component (str): component
        area (str): area
        itemid (int): itemid
        usescategories (int): view has categories
        categories (List[Category]): list of Category
    """
    component: str
    area: str
    itemid: int
    usescategories: int
    categories: List[Category]



from datetime import datetime
from typing import List, Optional, Union

from moodle import BaseMoodle
from moodle.core.course import CourseCapability
from moodle.core.user import User
from moodle.base.general import GeneralNameValue
from . import EditUserEnrolmentResponse, EnrolledUser, CapabilityEnrolledUser, EnrolmentMethod


class BaseEnrol(BaseMoodle):
    def edit_user_enrolment(
            self,
            courseid: int,
            ueid: int,
            status: int,
            timestart: Union[int, datetime] = 0,
            timeend: Union[int, datetime] = 0) -> EditUserEnrolmentResponse:
        """**DEPRECATED** Please do not call this function any more. External function that updates a given user enrolment

        Args:
            courseid (int): Course ID
            ueid (int): User enrolment ID
            status (int): Enrolment status
            timestart (Union[int, datetime], optional): Enrolment start timestamp. Defaults to 0.
            timeend (Union[int, datetime], optional): Enrolment end timestamp. Defaults to 0.

        Returns:
            EditUserEnrolmentResponse: Response
        """
        data = self.moodle.post(
            'core_enrol_edit_user_enrolment',
            courseid=courseid,
            ueid=ueid,
            status=status,
            timestart=timestart,
            timeend=timeend,
        )
        return self._tr(EditUserEnrolmentResponse, **data)

    def get_course_enrolment_methods(self,
                                     courseid: int) -> List[EnrolmentMethod]:
        """Get the list of course enrolment methods

        Args:
            courseid (int): Course id

        Returns:
            List[EnrolmentMethod]: list of EnrolmentMethod
        """
        datas = self.moodle.post(
            'core_enrol_get_course_enrolment_methods',
            courseid=courseid,
        )
        return [EnrolmentMethod(**data) for data in datas]  # type: ignore

    def get_enrolled_users(
            self, courseid: int,
            options: Optional[List[GeneralNameValue]]) -> List[EnrolledUser]:
        """Get enrolled users by course id.

        Args:
            courseid (int): course id
            options (Optional[List[GeneralNameValue]]): Option names:
                                                            * withcapability (string) return only users with this capability. This option requires 'moodle/role:review' on the course context.
                                                            * groupid (integer) return only users in this group id. If the course has groups enabled and this param isn't defined, returns all the viewable users. This option requires 'moodle/site:accessallgroups' on the course context if the user doesn't belong to the group.
                                                            * onlyactive (integer) return only users with active enrolments and matching time restrictions. This option requires 'moodle/course:enrolreview' on the course context.
                                                            * userfields ('string, string, ...') return only the values of these user fields.
                                                            * limitfrom (integer) sql limit from.
                                                            * limitnumber (integer) maximum number of returned users.
                                                            * sortby (string) sort by id, firstname or lastname. For ordering like the site does, use siteorder.
                                                            * sortdirection (string) ASC or DESC

        Returns:
            List[EnrolledUser]: list of EnrolledUser
        """
        data = self.moodle.post(
            'core_enrol_get_enrolled_users',
            courseid=courseid,
            options=options,
        )
        return [EnrolledUser(**dat) for dat in data]  # type: ignore

    def get_enrolled_users_with_capability(
            self, coursecapabilities: List[CourseCapability],
            options: List[GeneralNameValue]) -> List[CapabilityEnrolledUser]:
        """For each course and capability specified, return a list of the users that are enrolled in the course and have that capability

        Args:
            coursecapabilities (List[CourseCapability]): course id and associated capability name
            options (List[GeneralNameValue]): Option names:
                                                * groupid (integer) return only users in this group id. Requires 'moodle/site:accessallgroups' .
                                                * onlyactive (integer) only users with active enrolments. Requires 'moodle/course:enrolreview' .
                                                * userfields ('string, string, ...') return only the values of these user fields.
                                                * limitfrom (integer) sql limit from.
                                                * limitnumber (integer) max number of users per course and capability.

        Returns:
            List[CapabilityEnrolledUser]: list of the users that are enrolled in the course and have that capability
        """
        data = self.moodle.post(
            'core_enrol_get_enrolled_users_with_capability',
            coursecapabilities=coursecapabilities,
            options=options,
        )
        return [CapabilityEnrolledUser(**dat) for dat in data]  # type: ignore

    def get_potential_users(self, courseid: int, enrolid: int, search: str,
                            searchanywhere: int, page: int,
                            perpage: int) -> List[User]:
        """Get the list of potential users to enrol

        Args:
            courseid (int): course id
            enrolid (int): enrolment id
            search (str): query
            searchanywhere (int): find a match anywhere, or only at the beginning
            page (int): Page number
            perpage (int): Number per page

        Returns:
            List[User]: list of User
        """
        data = self.moodle.post(
            'core_enrol_get_potential_users',
            courseid=courseid,
            enrolid=enrolid,
            search=search,
            searchanywhere=searchanywhere,
            page=page,
            perpage=perpage,
        )
        return [User(**dat) for dat in data]  # type: ignore

    def get_users_courses(self):
        data = self.moodle.post('core_enrol_get_users_courses')
        return data

    def search_users(self):
        data = self.moodle.post('core_enrol_search_users')
        return data

    def submit_user_enrolment_form(self):
        data = self.moodle.post('core_enrol_submit_user_enrolment_form')
        return data

    def unenrol_user_enrolment(self):
        data = self.moodle.post('core_enrol_unenrol_user_enrolment')
        return data



from .edit_user_enrolment import EditUserEnrolmentResponse
from .enrolled_user import EnrolledUser
from .capability_enrolled_user import CapabilityEnrolledUser
from .enrolment_method import EnrolmentMethod

from .base import BaseEnrol

__all__ = [
    'EditUserEnrolmentResponse',
    'EnrolledUser',
    'CapabilityEnrolledUser',
    'EnrolmentMethod',
    'BaseEnrol',
]



from typing import Optional

from moodle.attr import dataclass
from moodle.base.general import GeneralTypeValue


@dataclass
class UserCustomField(GeneralTypeValue):
    """User custom fields (also known as user profil fields)

    Args:
        type (str): The name of the custom field
        value (str): The value of the custom field
        name (Optional[str]): The name of the custom field
        shortname (Optional[str]): The shortname of the custom field - to be able to build the field class in the code
    """
    name: Optional[str] = None
    shortname: Optional[str] = None



from .agree_site_policy_response import AgreeSitePolicyResponse
from .criteria import Criteria
from .enrolled_course import EnrolledCourse
from .user_custom_field import UserCustomField
from .user_preference import UserPreference
from .user import User
from .user_group import UserGroup
from .user_role import UserRole
from .user_list import UserList
from .user_profile import UserProfile
from .get_users_response import GetUsersResponse
from .create_user import CreateUser

from .base import BaseUser

__all__ = [
    'AgreeSitePolicyResponse',
    'Criteria',
    'EnrolledCourse',
    'UserCustomField',
    'UserPreference',
    'CreateUser',
    'User',
    'UserGroup',
    'UserRole',
    'UserList',
    'UserProfile',
    'GetUsersResponse',
    'BaseUser',
]



from moodle.attr import dataclass


@dataclass
class UserRole:
    """
    Args:
        roleid (int): role id
        name (str): role name
        shortname (str): role shortname
        sortorder (int): role sortorder
    """
    roleid: int
    name: str
    shortname: str
    sortorder: int



from typing import List

from moodle import BaseMoodle
from moodle.base.general import GeneralNameValue


class BaseWorkshop(BaseMoodle):
    def add_submission(self,
                       workshopid: int,
                       title: str,
                       content: str = '',
                       contentformat: int = 0,
                       inlineattachmentsid: int = 0,
                       attachmentsid: int = 0):
        data = self.moodle.post(
            'mod_workshop_add_submission',
            workshopid=workshopid,
            title=title,
            content=content,
            contentformat=contentformat,
            inlineattachmentsid=inlineattachmentsid,
            attachmentsid=attachmentsid,
        )
        return data

    def delete_submission(self, submissionid: int):
        data = self.moodle.post(
            'mod_workshop_delete_submission',
            submissionid=submissionid,
        )
        return data

    def evaluate_assessment(self,
                            assessmentid: int,
                            feedbacktext: str = '',
                            feedbackformat: int = 0,
                            weight: int = 1,
                            gradinggradeover: str = ''):
        data = self.moodle.post(
            'mod_workshop_evaluate_assessment',
            assessmentid=assessmentid,
            feedbacktext=feedbacktext,
            feedbackformat=feedbackformat,
            weight=weight,
            gradinggradeover=gradinggradeover,
        )
        return data

    def evaluate_submission(self,
                            submissionid: int,
                            feedbacktext: str = '',
                            feedbackformat: int = 0,
                            published: int = None,
                            gradeover: str = ''):
        data = self.moodle.post(
            'mod_workshop_evaluate_submission',
            submissionid=submissionid,
            feedbacktext=feedbacktext,
            feedbackformat=feedbackformat,
            published=published or '',
            gradeover=gradeover,
        )
        return data

    def get_assessment(self, assessmentid: int):
        data = self.moodle.post(
            'mod_workshop_get_assessment',
            assessmentid=assessmentid,
        )
        return data

    def get_assessment_form_definition(self,
                                       assessmentid: int,
                                       mode: str = 'assessment'):
        data = self.moodle.post(
            'mod_workshop_get_assessment_form_definition',
            assessmentid=assessmentid,
            mode=mode,
        )
        return data

    def get_grades(self, workshopid: int, userid: int = 0):
        data = self.moodle.post(
            'mod_workshop_get_grades',
            workshopid=workshopid,
            userid=userid,
        )
        return data

    def get_grades_report(self,
                          workshopid: int,
                          groupid: int = 0,
                          sortby: str = 'lastname',
                          sortdirection: str = 'ASC',
                          page: int = 0,
                          perpage: int = 0):
        data = self.moodle.post(
            'mod_workshop_get_grades_report',
            workshopid=workshopid,
            groupid=groupid,
            sortby=sortby,
            sortdirection=sortdirection,
            page=page,
            perpage=perpage,
        )
        return data

    def get_reviewer_assessments(self, workshopid: int, userid: int = 0):
        data = self.moodle.post(
            'mod_workshop_get_reviewer_assessments',
            workshopid=workshopid,
            userid=userid,
        )
        return data

    def get_submission(self, submissionid: int):
        data = self.moodle.post(
            'mod_workshop_get_submission',
            submissionid=submissionid,
        )
        return data

    def get_submission_assessments(self, submissionid: int):
        data = self.moodle.post(
            'mod_workshop_get_submission_assessments',
            submissionid=submissionid,
        )
        return data

    def get_submissions(self,
                        workshopid: int,
                        userid: int = 0,
                        groupid: int = 0,
                        page: int = 0,
                        perpage: int = 0):
        data = self.moodle.post(
            'mod_workshop_get_submissions',
            workshopid=workshopid,
            userid=userid,
            groupid=groupid,
            page=page,
            perpage=perpage,
        )
        return data

    def get_user_plan(self, workshopid: int, userid: int = 0):
        data = self.moodle.post(
            'mod_workshop_get_user_plan',
            workshopid=workshopid,
            userid=userid,
        )
        return data

    def get_workshop_access_information(self, workshopid: int):
        data = self.moodle.post(
            'mod_workshop_get_workshop_access_information',
            workshopid=workshopid,
        )
        return data

    def get_workshops_by_courses(self, courseids: List[int]):
        data = self.moodle.post(
            'mod_workshop_get_workshops_by_courses',
            courseids=courseids,
        )
        return data

    def update_assessment(self, assessmentid: int,
                          data: List[GeneralNameValue]):
        data = self.moodle.post(
            'mod_workshop_update_assessment',
            assessmentid=assessmentid,
            data=data,
        )
        return data

    def update_submission(self,
                          submissionid: int,
                          title: str,
                          content: str = '',
                          contentformat: int = 0,
                          inlineattachmentsid: int = 0,
                          attachmentsid: int = 0):
        data = self.moodle.post(
            'mod_workshop_update_submission',
            submissionid=submissionid,
            title=title,
            content=content,
            contentformat=contentformat,
            inlineattachmentsid=inlineattachmentsid,
            attachmentsid=attachmentsid,
        )
        return data

    def view_submission(self, submissionid: int):
        data = self.moodle.post(
            'mod_workshop_view_submission',
            submissionid=submissionid,
        )
        return data

    def view_workshop(self, workshopid: int):
        data = self.moodle.post(
            'mod_workshop_view_workshop',
            workshopid=workshopid,
        )
        return data



from moodle.base.general import GeneralResultError


class EditUserEnrolmentResponse(GeneralResultError):
    """[summary]

    Args:
        result (int): True if the user's enrolment was successfully updated
        errors (List[GeneralKeyMessage]): List of validation errors
    """
    pass



from moodle.attr import dataclass


@dataclass
class InstanceInfo:
    """Instance Info

    Args:
        id (int): Id of course enrolment instance
        courseid (int): Id of course
        type (str): Type of enrolment plugin
        name (str): Name of enrolment plugin
        status (str): Status of enrolment plugin
        passwordrequired (str): Password required for enrolment
    """
    id: int
    courseid: int
    type: str
    name: str
    status: str
    passwordrequired: str



from moodle.attr import dataclass


@dataclass
class UserCustomField:
    """UserCustomField
    Args:
        type (str): The type of the custom field
        name (str): The name of the custom field
        value (str): Custom field value, can be an encoded json if required
    """
    type: str
    name: str
    value: str



from typing import List

from moodle import BaseMoodle
from moodle.base.general import GeneralNameValue


class BaseData(BaseMoodle):
    def add_entry(self, databaseid: int, groupid: int, data: List[dict]):
        data = self.moodle.post(
            'mod_data_add_entry',
            databaseid=databaseid,
            groupid=groupid,
            data=data,
        )
        return data

    def approve_entry(self, entryid: int, approve: int = 1):
        data = self.moodle.post(
            'mod_data_approve_entry',
            entryid=entryid,
            approve=approve,
        )
        return data

    def delete_entry(self, entryid: int):
        data = self.moodle.post(
            'mod_data_delete_entry',
            entryid=entryid,
        )
        return data

    def get_data_access_information(self, databaseid: int, groupid: int = 0):
        data = self.moodle.post(
            'mod_data_get_data_access_information',
            databaseid=databaseid,
            groupid=groupid,
        )
        return data

    def get_databases_by_courses(self, courseids: List[int]):
        data = self.moodle.post(
            'mod_data_get_databases_by_courses',
            courseids=courseids,
        )
        return data

    def get_entries(self,
                    databaseid: int,
                    groupid: int = 0,
                    returncontents: int = None,
                    sort: int = None,
                    page: int = 0,
                    perpage: int = 0):
        data = self.moodle.post(
            'mod_data_get_entries',
            databaseid=databaseid,
            groupid=groupid,
            returncontents=returncontents or '',
            sort=sort,
            page=page,
            perpage=perpage,
        )
        return data

    def get_entry(self, entryid: int, returncontents: int = None):
        data = self.moodle.post(
            'mod_data_get_entry',
            entryid=entryid,
            returncontents=returncontents or '',
        )
        return data

    def get_fields(self, databaseid: int):
        data = self.moodle.post(
            'mod_data_get_fields',
            databaseid=databaseid,
        )
        return data

    def search_entries(self,
                       databaseid: int,
                       groupid: int = 0,
                       returncontents: int = None,
                       search: str = '',
                       advsearch: List[GeneralNameValue] = None,
                       sort: int = None,
                       order: str = None,
                       page: int = 0,
                       perpage: int = 0):
        data = self.moodle.post(
            'mod_data_search_entries',
            databaseid=databaseid,
            groupid=groupid,
            returncontents=returncontents or '',
            search=search,
            advsearch=advsearch or list(),
            sort=sort,
            order=order,
            page=page,
            perpage=perpage,
        )
        return data

    def update_entry(self, entryid: int, data: List[dict]):
        data = self.moodle.post(
            'mod_data_update_entry',
            entryid=entryid,
            data=data,
        )
        return data

    def view_database(self, databaseid: int):
        data = self.moodle.post(
            'mod_data_view_database',
            databaseid=databaseid,
        )
        return data



from moodle import BaseMoodle
from . import InstanceInfoResponse


class BaseGuest(BaseMoodle):
    def get_instance_info(self, instanceid: int) -> InstanceInfoResponse:
        data = self.moodle.post(
            'enrol_guest_get_instance_info',
            instanceid=instanceid,
        )
        return self._tr(InstanceInfoResponse, **data)



from moodle.attr import dataclass


@dataclass
class EnrolledCourse:
    """
    id (int): Id of the course
    fullname (str): Fullname of the course
    shortname (str): Shortname of the course
    """
    id: int
    fullname: str
    shortname: str



from moodle.base.general import GeneralStatus


class AgreeSitePolicyResponse(GeneralStatus):
    """Agree the site policy for the current user.

    Args:
        status (int): Status: true only if we set the policyagreed to 1 for the user
        warnings (List[Warning]): list of warnings
    """
    pass



from typing import List, Optional
from moodle import BaseMoodle
from moodle.utils.typing import Array
from . import SignupSetting, SignupUserResponse, UserCustomField


class BaseEmail(BaseMoodle):
    def get_signup_setting(self) -> SignupSetting:
        """Get the signup required settings and profile fields.

        Returns:
            SignupSetting: Signup required settings and profile fields
        """
        data = self.moodle.get('auth_email_get_signup_setting')
        return self._tr(SignupSetting, **data)

    def signup_user(self,
                    username: str,
                    password: str,
                    firstname: str,
                    lastname: str,
                    email: str,
                    city: str = '',
                    country: str = '',
                    recaptchachallengehash: str = '',
                    recaptcharesponse: str = '',
                    customprofilefields: Optional[
                        List[UserCustomField]] = None,
                    redirect: str = '') -> SignupUserResponse:
        """Adds a new user (pendingto be confirmed) in the site.

        Args:
            username (str): Username
            password (str): Plain text password
            firstname (str): The first name(s) of the user
            lastname (str): The family name of the user
            email (str): A valid and unique email address
            city (str, optional): Home city of the user. Defaults to ''.
            country (str, optional): Home country code. Defaults to ''.
            recaptchachallengehash (str, optional): Recaptcha challenge hash. Defaults to ''.
            recaptcharesponse (str, optional): Recaptcha response. Defaults to ''.
            customprofilefields (Optional[ List[UserCustomField]], optional): User custom fields (also known as user profile fields). Defaults to None.
            redirect (str, optional): Redirect the user to this site url after confirmation.. Defaults to ''.

        Returns:
            SignupUserResponse: Response
        """
        customprofilefields = Array(
            customprofilefields if customprofilefields else [])
        customprofilefields.name = 'customprofilefields'
        data = self.moodle.get(
            'auth_email_signup_user',
            username=username,
            password=password,
            firstname=firstname,
            lastname=lastname,
            email=email,
            city=city,
            country=country,
            recaptchachallengehash=recaptchachallengehash,
            recaptcharesponse=recaptcharesponse,
            customprofilefields=customprofilefields,
            redirect=redirect,
        )
        return self._tr(SignupUserResponse, **data)



from typing import List, Optional

from moodle import BaseMoodle
from . import (
    Content,
    FunctionsResponses,
    Key,
    MobileConfig,
    MobilePlugin,
    MobilePublicConfig,
    TokenPrivateToken,
)
from .validated_key import ValidatedKey


class BaseMobile(BaseMoodle):
    def call_external_functions(
            self,
            requests: List[FunctionsResponses.Request]) -> FunctionsResponses:
        """Call multiple external functions and return all responses.

        Args:
            requests (List[FunctionsResponses.Request]): Functions requests

        Returns:
            FunctionsResponses: Functions Response
        """
        res = self.moodle.post('tool_mobile_call_external_functions')
        return self._tr(FunctionsResponses, **res)

    def get_autologin_key(self, privatetoken: str) -> Key:
        """Creates an auto-login key for the current user.
        Is created only in https sites and is restricted by time, ip address
        and only works if the request comes from the Moodle mobile or desktop app.

        Args:
            privatetoken (str): Private token, usually generated by login/token.php

        Returns:
            Key: Key object containing auto-login
        """
        res = self.moodle.post('tool_mobile_get_autologin_key',
                               privatetoken=privatetoken)
        return self._tr(Key, **res)

    def get_config(self, section: Optional[str] = None) -> MobileConfig:
        """Returns a list of the site configurations, filtering by section.

        Args:
            section (str, optional): Settings section name. Defaults to None.

        Returns:
            MobileConfig: Returns a list of the site configurations, filtering by section.
        """
        res = self.moodle.get('tool_mobile_get_config', section=section or '')
        return self._tr(MobileConfig, **res)

    def get_content(self,
                    component: str,
                    method: str,
                    args: Optional[Content.Args] = None) -> Content:
        """Get a piece of content to be displayed in the Mobile app.

        Args:
            component (str): Component where the class is e.g. mod_assign.
            method (str): Method to execute in class \\$component\\output\\mobile.
            args (Optional[Content.Args], optional): Args for the method are optional.. Defaults to None.

        Returns:
            Content: Returns a piece of content to be displayed in the Mobile app.
        """
        res = self.moodle.post(
            'tool_mobile_get_content',
            component=component,
            method=method,
            args=args,
        )
        return self._tr(Content, **res)

    def get_plugins_supporting_mobile(self) -> List[MobilePlugin]:
        """Get a list of Moodle plugins supporting the mobile app.

        Returns:
            List[MobilePlugin]: Returns a list of Moodle plugins supporting the mobile app.
        """
        res = self.moodle.get('tool_mobile_get_plugins_supporting_mobile')
        results: List[MobilePlugin] = list()
        if not res:
            return results
        plugins = res['plugins']
        for plugin in plugins:
            results.append(MobilePlugin(**plugin))  # type: ignore
        return results

    def get_public_config(self) -> MobilePublicConfig:
        """Get a list of the site public settings, those not requiring authentication.

        Returns:
            MobilePublicConfig: Returns a list of the site public settings, those not requiring authentication.
        """
        data = self.moodle.get('tool_mobile_get_public_config')
        return self._tr(MobilePublicConfig, **data)

    def get_tokens_for_qr_login(self, qrloginkey: str,
                                userid: int) -> TokenPrivateToken:
        """Returns a WebService token (and private token) for QR login.

        Args:
            qrloginkey (str): The user key for validating the request.
            userid (int): The user the key belongs to.

        Returns:
            TokenPrivateToken: WebService token (and private token)
        """
        data = self.moodle.post(
            'tool_mobile_get_tokens_for_qr_login',
            qrloginkey=qrloginkey,
            userid=userid,
        )
        return self._tr(TokenPrivateToken, **data)

    def validate_subscription_key(self, key: str) -> ValidatedKey:
        data = self.moodle.post(
            'tool_mobile_validate_subscription_key',
            key=key,
        )
        return self._tr(ValidatedKey, **data)



from moodle import BaseMoodle
from moodle.base.general import GeneralSuccess
from . import RequestPasswordResetResponse


class BaseAuth(BaseMoodle):
    def confirm_user(self, username: str, secret: str) -> GeneralSuccess:
        """Confirm a user account.

        Args:
            username (str): User name
            secret (str): Confirmation secret

        Returns:
            GeneralSuccess: True if the user was confirmed, false if he was already confirmed
        """
        data = self.moodle.post(
            'core_auth_confirm_user',
            username=username,
            secret=secret,
        )
        return self._tr(GeneralSuccess, **data)

    def is_age_digital_consent_verification_enabled(self) -> GeneralSuccess:
        """Checks if age digital consent verification is enabled.

        Returns:
            GeneralSuccess: True if digital consent verification is enabled, false otherwise.
        """
        data = self.moodle.post(
            'core_auth_is_age_digital_consent_verification_enabled')
        return self._tr(GeneralSuccess, **data)

    def is_minor(self, age: int, country: str) -> GeneralSuccess:
        """Requests a check if a user is a digital minor.

        Args:
            age (int): Age
            country (str): Country of residence

        Returns:
            GeneralSuccess: True if the user is considered to be a digital minor, false if not
        """
        data = self.moodle.post(
            'core_auth_is_minor',
            age=age,
            country=country,
        )
        return self._tr(GeneralSuccess, **data)

    def request_password_reset(
            self,
            username: str = '',
            email: str = '') -> RequestPasswordResetResponse:
        """Requests a password reset.

        Args:
            username (str, optional): User name. Defaults to ''.
            email (str, optional): User email. Defaults to ''.

        Returns:
            RequestPasswordResetResponse: Response
        """
        data = self.moodle.post(
            'core_auth_request_password_reset',
            username=username,
            email=email,
        )
        return self._tr(RequestPasswordResetResponse, **data)

    def resend_confirmation_email(self,
                                  username: str,
                                  password: str,
                                  redirect: str = '') -> GeneralSuccess:
        data = self.moodle.post(
            'core_auth_resend_confirmation_email',
            username=username,
            password=password,
            redirect=redirect,
        )
        return self._tr(GeneralSuccess, **data)



from typing import List, Optional

from moodle import BaseMoodle, MoodleWarning
from . import (
    AgreeSitePolicyResponse,
    Criteria,
    CreateUser,
    GetUsersResponse,
    UserList,
    UserProfile,
)


class BaseUser(BaseMoodle):
    def add_user_device(self, appid: str, name: str, model: str, platform: str,
                        version: str, pushid: str,
                        uuid: str) -> List[Optional[List[MoodleWarning]]]:
        """Store mobile user devices information for PUSH Notifications.

        Args:
            appid (str): the app id, usually something like com.moodle.moodlemobile
            name (str): the device name, 'occam' or 'iPhone' etc.
            model (str): the device model 'Nexus4' or 'iPad1,1' etc.
            platform (str): the device platform 'iOS' or 'Android' etc.
            version (str): the device platform 'iOS' or 'Android' etc.
            pushid (str): the device version '6.1.2' or '4.2.2' etc.
            uuid (str): the device PUSH token/key/identifier/registration id

        Returns:
            List[Optional[List[Warning]]]: Response
        """
        results: List[Optional[List[MoodleWarning]]] = list()
        data = self.moodle.post(
            'core_user_add_user_device',
            appid=appid,
            name=name,
            model=model,
            platform=platform,
            version=version,
            pushid=pushid,
            uuid=uuid,
        )
        if not data:
            return results
        for dat in data:
            results.append([MoodleWarning(**da) for da in dat])  # type: ignore
        return results

    def add_user_private_files(self, draftid: int):
        """Copy files from a draft area to users private files area.

        Args:
            draftid (int): draft area id

        Returns:
            None: No response
        """
        data = self.moodle.post(
            'core_user_add_user_private_files',
            draftid=draftid,
        )
        return data

    def agree_site_policy(self) -> AgreeSitePolicyResponse:
        """Agree the site policy for the current user.

        Returns:
            AgreeSitePolicyResponse: Response
        """
        data = self.moodle.post('core_user_agree_site_policy')
        return self._tr(AgreeSitePolicyResponse, **data)

    def create_users(self,
                     users: List[CreateUser]) -> List[CreateUser.Response]:
        """Create users.

        Args:
            users (List[CreateUser]): list of CreateUser

        Returns:
            List[CreateUser.Response]: list of created user id and user name
        """
        data = self.moodle.post(
            'core_user_create_users',
            users=users,
        )
        return [CreateUser.Response(**dat) for dat in data]  # type: ignore

    def delete_users(self, userids: List[int]) -> None:
        """Delete users.

        Args:
            userids (List[int]): list of user ID

        Returns:
            None: None
        """
        data = self.moodle.post(
            'core_user_delete_users',
            userids=userids,
        )
        return data

    def get_course_user_profiles(
            self, userlist: List[UserList]) -> List[UserProfile]:
        """Get course user profiles (each of the profils matching a course id and a user id),.

        Args:
            userlist (List[UserList]): list of UserList or dict containing userid and courseid

        Returns:
            List[UserProfile]: list of UserProfile
        """
        data = self.moodle.post(
            'core_user_get_course_user_profiles',
            userlist=userlist,
        )
        return [UserProfile(**dat) for dat in data]  # type: ignore

    def get_private_files_info(self):
        data = self.moodle.post('core_user_get_private_files_info')
        return data

    def get_user_preferences(self):
        data = self.moodle.post('core_user_get_user_preferences')
        return data

    def get_users(self, criteria: List[Criteria]) -> GetUsersResponse:
        """search for users matching the parameters

        Args:
            criteria (List[Criteria]): the key/value pairs to be considered in user search.
                                        Values can not be empty. Specify different keys only once (fullname => 'user1', auth => 'manual', ...) - key occurences are forbidden.
                                        The search is executed with AND operator on the criterias. Invalid criterias (keys) are ignored, the search is still executed on the valid criterias.
                                        You can search without criteria, but the function is not designed for it. It could very slow or timeout.
                                        The function is designed to search some specific users.

        Returns:
            GetUsersResponse: Response
        """
        data = self.moodle.post(
            'core_user_get_users',
            criteria=criteria,
        )
        return self._tr(GetUsersResponse, **data)

    def get_users_by_field(self):
        data = self.moodle.post('core_user_get_users_by_field')
        return data

    def remove_user_device(self):
        data = self.moodle.post('core_user_remove_user_device')
        return data

    def set_user_preferences(self):
        data = self.moodle.post('core_user_set_user_preferences')
        return data

    def update_picture(self):
        data = self.moodle.post('core_user_update_picture')
        return data

    def update_user_preferences(self):
        data = self.moodle.post('core_user_update_user_preferences')
        return data

    def update_users(self):
        data = self.moodle.post('core_user_update_users')
        return data

    def view_user_list(self):
        data = self.moodle.post('core_user_view_user_list')
        return data

    def view_user_profile(self):
        data = self.moodle.post('core_user_view_user_profile ')
        return data



from moodle.base.general import GeneralValidated


class ValidatedKey(GeneralValidated):
    """ValidatedKey
    Args:
        validated (int): Whether the key is validated or not.
        warnings (List[Warning]): list of warnings
    """
    pass



from moodle.attr import dataclass
from typing import Any, Optional
from requests.exceptions import RequestException


@dataclass
class MoodleException(Exception):
    errorcode: Optional[Any] = ''
    exception: Optional[Any] = ''
    message: Optional[str] = ''

    def __str__(self):
        return self.message or self.exception or self.errorcode


class BaseException(Exception):
    message: str = ''

    def __post_init__(self) -> None:
        self.message = self.message.capitalize()

    def __str__(self) -> str:
        return self.message


@dataclass
class EmptyResponseException(BaseException):
    message: str = 'Empty response from server!'


@dataclass
class InvalidCredentialException(BaseException):
    message: str = 'Wrong username or password!'


@dataclass
class NetworkMoodleException(BaseException):
    """Moodle wrapper for network related network error
    """
    exception: Optional[RequestException] = None
    message: str = 'A Network error occurred'



from moodle.attr import dataclass
from typing import Any


@dataclass
class GeneralKeyValue:
    """GeneralKeyValue

    Args:
        key (str): ...
        value (Any): ...
    """
    key: str
    value: Any



from typing import List

from moodle.attr import dataclass, fields
from .notification_component import NotificationComponent


@dataclass
class PreferenceComponent:
    displayname: str
    notification: List[NotificationComponent] = fields(NotificationComponent)



from typing import Optional, Type, TypeVar, TYPE_CHECKING
if TYPE_CHECKING:
    from moodle import Moodle

T = TypeVar('T')


class MoodleObject(object):
    wsfunction: str = ''

    def __init__(self, *args, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

    def __post_init__(self):
        self._moodle: Optional['Moodle'] = None

    @property
    def moodle(self) -> Optional['Moodle']:
        return self._moodle

    @classmethod
    def from_data(cls: Type[T], data: dict, moodle: 'Moodle') -> T:
        self = cls(**data)  # type: ignore
        setattr(self, '_moodle', moodle)
        return self

    def __str__(self) -> str:
        return getattr(self, 'name', repr(self))



from moodle.base.general import GeneralKeyValue


class Criteria(GeneralKeyValue):
    """the key/value pairs to be considered in user search. Values can not be empty.
        Specify different keys only once (fullname => 'user1', auth => 'manual', ...) -
        key occurences are forbidden.
        The search is executed with AND operator on the criterias. Invalid criterias (keys) are ignored,
        the search is still executed on the valid criterias.
        You can search without criteria, but the function is not designed for it.
        It could very slow or timeout. The function is designed to search some specific users.

    Args:
        key (str): the user column to search, expected keys (value format) are:
                            "id" (int) matching user id,
                            "lastname" (string) user last name (Note: you can use % for searching but it may be considerably slower!),
                            "firstname" (string) user first name (Note: you can use % for searching but it may be considerably slower!),
                            "idnumber" (string) matching user idnumber,
                            "username" (string) matching user username,
                            "email" (string) user email (Note: you can use % for searching but it may be considerably slower!),
                            "auth" (string) matching user auth plugin
        value (int, str): the value to search
    """
    pass



from typing import List

from moodle import MoodleWarning
from moodle.attr import dataclass, fields


@dataclass
class ViewEntry:
    """the blog_entries_viewed event.
    Constructor arguments:
    params: status (bool): status: true if success
    params: warnings (List[Warning]): list of warnings
    """
    status: bool
    warnings: List[MoodleWarning] = fields(MoodleWarning)



from moodle import BaseMoodle
from moodle.utils.decorator import lazy
from . import (
    BaseAssign,
    BaseChat,
    BaseData,
    BaseFolder,
    BaseForum,
    BaseLesson,
    BasePage,
    BaseResource,
    BaseUrl,
    BaseWorkshop,
)


class Mod(BaseMoodle):
    @property  # type: ignore
    @lazy
    def assign(self) -> BaseAssign:
        return BaseAssign(self.moodle)

    @property  # type: ignore
    @lazy
    def chat(self) -> BaseChat:
        return BaseChat(self.moodle)

    @property  # type: ignore
    @lazy
    def data(self) -> BaseData:
        return BaseData(self.moodle)

    @property  # type: ignore
    @lazy
    def folder(self) -> BaseFolder:
        return BaseFolder(self.moodle)

    @property  # type: ignore
    @lazy
    def forum(self) -> BaseForum:
        return BaseForum(self.moodle)

    @property  # type: ignore
    @lazy
    def lesson(self) -> BaseLesson:
        return BaseLesson(self.moodle)

    @property  # type: ignore
    @lazy
    def page(self) -> BasePage:
        return BasePage(self.moodle)

    @property  # type: ignore
    @lazy
    def resource(self) -> BaseResource:
        return BaseResource(self.moodle)

    @property  # type: ignore
    @lazy
    def url(self) -> BaseUrl:
        return BaseUrl(self.moodle)

    @property  # type: ignore
    @lazy
    def workshop(self) -> BaseWorkshop:
        return BaseWorkshop(self.moodle)



from .assign import BaseAssign
from .chat import BaseChat
from .data import BaseData
from .folder import BaseFolder
from .forum import BaseForum
from .lesson import BaseLesson
from .page import BasePage
from .resource import BaseResource
from .url import BaseUrl
from .workshop import BaseWorkshop

from .mod import Mod

__all__ = [
    'BaseAssign',
    'BaseChat',
    'BaseData',
    'BaseFolder',
    'BaseForum',
    'BaseLesson',
    'BasePage',
    'BaseResource',
    'BaseUrl',
    'BaseWorkshop',
    'Mod',
]



from typing import List

from moodle import BaseMoodle
from .filepath_filename import FilepathFilename
from .parent_paths import ParentPaths


class BaseFiles(BaseMoodle):
    def delete_draft_files(self, draftitemid: int,
                           files: List[FilepathFilename]) -> ParentPaths:
        """Delete the indicated files (or directories) from a user draft file area.

        Args:
            draftitemid (int): Item id of the draft file area
            files (List[FilepathFilename]): Files or directories to be deleted.

        Returns:
            ParentPaths: Path(s) to parent directory of the deleted file(s)
        """
        data = self.moodle.post('core_files_delete_draft_files')
        return self._tr(ParentPaths, **data)

    def get_files(self,
                  contextid: int,
                  component: str,
                  filearea: str,
                  itemid: int,
                  filepath: str,
                  filename: str,
                  modified: int = None,
                  contextlevel: str = None,
                  instanceid: int = None):
        data = self.moodle.post(
            'core_files_get_files',
            contextid=contextid,
            component=component,
            filearea=filearea,
            itemid=itemid,
            filepath=filepath,
            filename=filename,
            modified=modified,
            contextlevel=contextlevel,
            instanceid=instanceid,
        )
        return data

    def upload(self,
               component: str,
               filearea: str,
               itemid: int,
               filepath: str,
               filename: str,
               filecontent: str,
               contextid: int = None,
               contextlevel: str = None,
               instanceid: int = None):
        data = self.moodle.post(
            'core_files_upload',
            component=component,
            filearea=filearea,
            itemid=itemid,
            filepath=filepath,
            filename=filename,
            filecontent=filecontent,
            contextid=contextid,
            contextlevel=contextlevel,
            instanceid=instanceid,
        )
        return data



from moodle.attr import dataclass
from typing import List, Optional
from moodle import MoodleWarning, ResponsesFactory


@dataclass
class File:
    """File
    Args:
        filename (Optional[str]): File name.
        filepath (Optional[str]): File path.
        filesize (Optional[int]): File size.
        fileurl (Optional[str]): Downloadable file url.
        timemodified (Optional[int]): Time modified.
        mimetype (Optional[str]): File mime type.
        isexternalfile (Optional[int]): Whether is an external file.
        repositorytype (Optional[str]): The repository type for external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class Folder:
    """Folder
    Args:
        id (int): Module id
        coursemodule (int): Course module id
        course (int): Course id
        name (str): Page name
        intro (str): Summary
        introformat (int): Default untuk "1" intro format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        introfiles (List[File]): Files in the introduction text
        revision (int): Incremented when after each file changes, to avoid cache
        timemodified (int): Last time the folder was modified
        display (int): Display type of folder contents on a separate page or inline
        showexpanded (int): 1 = expanded, 0 = collapsed for sub-folders
        showdownloadfolder (int): Whether to show the download folder button
        section (int): Course section id
        visible (int): Module visibility
        groupmode (int): Group mode
        groupingid (int): Grouping id
    """
    id: int
    coursemodule: int
    course: int
    name: str
    intro: str
    introformat: int
    introfiles: List[File]
    revision: int
    timemodified: int
    display: int
    showexpanded: int
    showdownloadfolder: int
    section: int
    visible: int
    groupmode: int
    groupingid: int


@dataclass
class Folders(ResponsesFactory[Folder]):
    """Folders (list of Folder)
    Args:
        folders (List[Folder]): list of folders
        warnings (List[Warning]): list of warnings
    """
    folders: List[Folder]
    warnings: List[MoodleWarning]

    @property
    def items(self) -> List[Folder]:
        return self.folders



from moodle.attr import dataclass
from typing import List, Optional
from moodle import MoodleWarning


@dataclass
class ContactRequestRecord:
    """Request Record
    Constructor arguments:
        id (int): Message id
        userid (int): User from id
        requesteduserid (int): User to id
        timecreated (int): Time created
    """
    id: int
    userid: int
    requesteduserid: int
    timecreated: int


@dataclass
class ContactRequest:
    """Contact Request
    Constructor arguments:
        request (Optional[ContactRequestRecord]): request record
        warnings (List[Warning]): list of warnings
    """
    request: Optional[ContactRequestRecord]
    warnings: List[MoodleWarning]



from moodle.attr import dataclass
from typing import Optional


@dataclass
class StarredCourse:
    """
    Args:
        id (int): id
        fullname (str): fullname
        shortname (str): shortname
        idnumber (str): idnumber
        summary (str): summary
        summaryformat (int): summary format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        startdate (int): startdate
        enddate (int): enddate
        visible (int): visible
        fullnamedisplay (str): fullnamedisplay
        viewurl (str): viewurl
        courseimage (str): courseimage
        progress (Optional[int]): progress
        hasprogress (int): hasprogress
        isfavourite (int): isfavourite
        hidden (int): hidden
        timeaccess (Optional[int]): timeaccess
        showshortname (int): showshortname
        coursecategory (str): coursecategory
    """
    id: int
    fullname: str
    shortname: str
    idnumber: str
    summary: str
    summaryformat: int
    startdate: int
    enddate: int
    visible: int
    fullnamedisplay: str
    viewurl: str
    courseimage: str
    progress: Optional[int]
    hasprogress: int
    isfavourite: int
    hidden: int
    timeaccess: Optional[int]
    showshortname: int
    coursecategory: str



from moodle.attr import dataclass
from datetime import datetime
from typing import Optional, Union


@dataclass
class Enrolment:
    """Enrolment

    Args:
        roleid (int): Role to assign to the user
        userid (int): The user that is going to be enrolled
        courseid (int): The course to enrol the user role in
        timestart (Optional[Union[datetime, int]]): Timestamp when the enrolment start
        timeend (Optional[Union[datetime, int]]): Timestamp when the enrolment end
        suspend (Optional[int]): set to 1 to suspend the enrolment
    """
    roleid: int
    userid: int
    courseid: int
    timestart: Optional[Union[datetime, int]]
    timeend: Optional[Union[datetime, int]]
    suspend: Optional[int]



from typing import List

from moodle import ResponsesFactory
from moodle.attr import dataclass, fields
from . import User


@dataclass
class GetUsersResponse(ResponsesFactory[User]):
    users: List[User] = fields(User)
    warnings: List[Warning] = fields(Warning)

    @property
    def items(self) -> List[User]:
        return self.users



from moodle.attr import dataclass
from moodle import MoodleWarning
from typing import Optional, List


@dataclass
class ProfileField:
    """ProfileField
    Args:
        id (Optional[int]): Profile field id
        shortname (Optional[str]): Profile field shortname
        name (Optional[str]): Profield field name
        datatype (Optional[str]): Profield field datatype
        description (Optional[str]): Profield field description
        descriptionformat (int): description format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        categoryid (Optional[int]): Profield field category id
        categoryname (Optional[str]): Profield field category name
        sortorder (Optional[int]): Profield field sort order
        required (Optional[int]): Profield field required
        locked (Optional[int]): Profield field locked
        visible (Optional[int]): Profield field visible
        forceunique (Optional[int]): Profield field unique
        signup (Optional[int]): Profield field in signup form
        defaultdata (Optional[str]): Profield field default data
        defaultdataformat (int): defaultdata format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        param1 (Optional[str]): Profield field settings
        param2 (Optional[str]): Profield field settings
        param3 (Optional[str]): Profield field settings
        param4 (Optional[str]): Profield field settings
        param5 (Optional[str]): Profield field settings
    """
    id: Optional[int]
    shortname: Optional[str]
    name: Optional[str]
    datatype: Optional[str]
    description: Optional[str]
    descriptionformat: int
    categoryid: Optional[int]
    categoryname: Optional[str]
    sortorder: Optional[int]
    required: Optional[int]
    locked: Optional[int]
    visible: Optional[int]
    forceunique: Optional[int]
    signup: Optional[int]
    defaultdata: Optional[str]
    defaultdataformat: int
    param1: Optional[str]
    param2: Optional[str]
    param3: Optional[str]
    param4: Optional[str]
    param5: Optional[str]


@dataclass
class SignupSetting:
    """SignupSetting
    Args:
        namefields (List[str]): The order of the name fields
        passwordpolicy (Optional[str]): Password policy
        sitepolicy (Optional[str]): Site policy
        sitepolicyhandler (Optional[str]): Site policy handler
        defaultcity (Optional[str]): Default city
        country (Optional[str]): Default country
        profilefields (List[ProfileField]): Required profile fields
        recaptchapublickey (Optional[str]): Recaptcha public key
        recaptchachallengehash (Optional[str]): Recaptcha challenge hash
        recaptchachallengeimage (Optional[str]): Recaptcha challenge noscript image
        recaptchachallengejs (Optional[str]): Recaptcha challenge js url
        warnings (List[Warning]): list of warnings
    """
    namefields: List[str]
    passwordpolicy: Optional[str]
    sitepolicy: Optional[str]
    sitepolicyhandler: Optional[str]
    defaultcity: Optional[str]
    country: Optional[str]
    profilefields: List[ProfileField]
    recaptchapublickey: Optional[str]
    recaptchachallengehash: Optional[str]
    recaptchachallengeimage: Optional[str]
    recaptchachallengejs: Optional[str]
    warnings: List[MoodleWarning]



from typing import List

from moodle import BaseMoodle
from . import RecentlyAccessedItem, StarredCourse


class Block(BaseMoodle):
    def recentlyaccesseditems_get_recent_items(self,
                                               limit: int = 0
                                               ) -> List[RecentlyAccessedItem]:
        """List of items a user has accessed most recently.

        Args:
            limit (int, optional): result set limit. Defaults to 0.

        Returns:
            List[RecentlyAccessedItem]: The most recently accessed activities/resources by the logged user
        """
        data = self.moodle.post(
            'block_recentlyaccesseditems_get_recent_items',
            limit=limit,
        )
        return [RecentlyAccessedItem(**dat) for dat in data]  # type: ignore

    def starredcourses_get_starred_courses(self,
                                           limit: int = 0,
                                           offset: int = 0
                                           ) -> List[StarredCourse]:
        """Get users starred courses.

        Args:
            limit (int, optional): Limit. Defaults to 0.
            offset (int, optional): Offset. Defaults to 0.

        Returns:
            List[StarredCourse]: users starred courses
        """
        data = self.moodle.post(
            'block_starredcourses_get_starred_courses',
            limit=limit,
            offset=offset,
        )
        return [StarredCourse(**dat) for dat in data]  # type: ignore



from typing import Any, List

from moodle import BaseMoodle
from . import (
    BackupCompletionStatus,
    CopyData,
    CopyDataResponse,
)
from .table_row_data import TableRowData


class BaseBackup(BaseMoodle):
    def get_async_backup_links_backup(self, filename: str,
                                      contextid: int) -> TableRowData:
        """Gets the data to use when updating the status table row in the UI for when an async backup completes.

        Args:
            filename (str): Backup filename
            contextid (int): Context id

        Returns:
            TableRowData: Table row data.
        """
        data = self.moodle.post(
            'core_backup_get_async_backup_links_backup',
            filename=filename,
            contextid=contextid,
        )
        return self._tr(TableRowData, **data)

    def get_async_backup_links_restore(self, backupid: str,
                                       contextid: int) -> str:
        """Gets the data to use when updating the status table row in the UI for when an async restore completes.

        Args:
            backupid (str): Backup id
            contextid (int): Context id

        Returns:
            str: Restore url
        """
        data = self.moodle.post(
            'core_backup_get_async_backup_links_restore',
            backupid=backupid,
            contextid=contextid,
        )
        return dict(data).get('restoreurl', '')

    def get_async_backup_progress(
            self, backupids: List[str],
            contextid: int) -> List[BackupCompletionStatus]:
        """Get the progress of an Asyncronhous backup.

        Args:
            backupids (List[str]): Backup id(s) to get progress for
            contextid (int): Context id

        Returns:
            List[BackupCompletionStatus]: Backup data
        """
        data = self.moodle.post(
            'core_backup_get_async_backup_progress',
            backupids=backupids,
            contextid=contextid,
        )
        return [BackupCompletionStatus(**dat) for dat in data]  # type: ignore

    def get_copy_progress(self,
                          copies: List[CopyData]) -> List[CopyDataResponse]:
        """Gets the progress of course copy operations.

        Args:
            copies (List[CopyData]): Copy data

        Returns:
            List[CopyDataResponse]: Copy data completion statuses
        """
        data = self.moodle.post(
            'core_backup_get_copy_progress',
            copies=copies,
        )
        return [CopyDataResponse(**dat) for dat in data]  # type: ignore

    def submit_copy_form(self, jsonformdata: str) -> Any:
        """Handles ajax submission of course copy form.

        Args:
            jsonformdata (str): The data from the create copy form, encoded as a json array

        Returns:
            Any: JSON response.
        """
        data = self.moodle.post(
            'core_backup_submit_copy_form',
            jsonformdata=jsonformdata,
        )
        return data



from moodle.attr import dataclass
from typing import List

from moodle import MoodleWarning


@dataclass
class GeneralStatus:
    """GeneralStatus
    Args:
        success (int): True or False, nor 1 or 0
        warnings (List[Warning]): list of warnings
    """
    status: int
    warnings: List[MoodleWarning]



from typing import List

from moodle import MoodleWarning
from moodle.attr import dataclass, fields


@dataclass
class InstanceInfo:
    """Instance Info

    Args:
        id (int): Id of course enrolment instance
        courseid (int): Id of course
        type (str): Type of enrolment plugin
        name (str): Name of enrolment plugin
        status (int): Is the enrolment enabled?
        passwordrequired (int): Is a password required?
    """
    id: int
    courseid: int
    type: str
    name: str
    status: int
    passwordrequired: int


@dataclass
class InstanceInfoResponse:
    """InstanceInfoResponse

    Args:
        instanceinfo (InstanceInfo): Course enrolment instance
        warnings (List[MoodleWarning]): list of warnings
    """
    instanceinfo: InstanceInfo
    warnings: List[MoodleWarning] = fields(MoodleWarning)



from typing import List

from moodle import BaseMoodle
from . import Enrolment


class BaseManual(BaseMoodle):
    def enrol_users(self, enrolments: List[Enrolment]) -> None:
        """Manual enrol users

        Args:
            enrolments (List[Enrolment]): enrolments

        Returns:
            None: Nothing
        """
        data = self.moodle.post(
            'enrol_manual_enrol_users',
            enrolments=enrolments,
        )
        return data

    def unenrol_users(self, enrolments: List[Enrolment]) -> None:
        """Manual unenrol users

        Args:
            enrolments (List[Enrolment]): enrolments

        Returns:
            None: Nothing
        """
        data = self.moodle.post(
            'enrol_manual_unenrol_users',
            enrolments=enrolments,
        )
        return data



from attr import attrs, attrib, has, Attribute
from attr import asdict as asdict_attr
from datetime import datetime
from functools import partial
from typing import Any, Callable, List, Optional

from moodle.base.moodle_object import MoodleObject  # NOQA

COMMON: List = [str, int, datetime]


def fromtimestamp(d: str):
    if not isinstance(d, str):
        return d
    elif not d.isdigit():
        return d
    return datetime.fromtimestamp(float(d))


def hooq(cls: type, fields: List[Attribute]) -> List[Attribute]:
    results: List[Attribute] = list()
    return results


def serialize(inst=None,
              field: Attribute = None,
              value: Any = None,
              name: str = None) -> Any:
    if not value:
        return value
    if not name:
        name = getattr(field, 'name', '')
    if isinstance(value, list):
        out = {}
        for idx, val in enumerate(value):
            val = serialize(value=val)
            if isinstance(val, dict):
                for key, value in val.items():
                    out[f'{name}[{idx}][{key}]'] = val[key]
            else:
                out_key = name or ''
                # Check if data required name prefix
                if hasattr(value, 'name'):
                    out_key += f"[{getattr(value, 'name')}]"
                out_key += f'[{idx}]'
                out[out_key] = val
        return out
    elif isinstance(value, dict):
        out = {}
        for key, value in value.items():
            if isinstance(value, list):
                out.update(serialize(value=val, name=key))
            else:
                out[key] = value
        return out
    elif has(value):
        return asdict_attr(value, value_serializer=serialize)  # type: ignore
    elif isinstance(value, datetime):
        return datetime.timestamp(value)
    return value


def auto_convert(cls, fields):
    results = []
    for field in fields:
        if field.converter is not None:
            results.append(field)
            continue
        if field.type in {datetime, 'datetime'}:
            converter = fromtimestamp
        elif field.type == Optional[field.type]:

            def converter(d=None):
                return d
        else:
            converter = None
        results.append(field.evolve(converter=converter))
    return results


asdict: Callable = partial(
    asdict_attr,
    value_serializer=serialize,
)
dataclass: Callable = partial(
    attrs,
    auto_attribs=True,
    field_transformer=auto_convert,
)
sdataclass: Callable = partial(
    attrs,
    auto_attribs=True,
    field_transformer=auto_convert,
    slots=True,
)


def fields(converter: Callable):
    def conv(d: List[dict]):
        results: List = list()
        if not isinstance(d, list):
            return results
        for result in results:
            results.append(converter(**result))
        return results

    return attrib(converter=conv, factory=list)



from moodle import BaseMoodle
from . import SiteInfo


class BaseWebservice(BaseMoodle):
    def get_site_info(self) -> SiteInfo:
        data = self.moodle.get('core_webservice_get_site_info')
        if data:
            return self._tr(SiteInfo, **data)
        raise Exception('Info not found')



from .recently_accessed_item import RecentlyAccessedItem
from .starred_course import StarredCourse

from .block import Block

__all__ = [
    "RecentlyAccessedItem",
    "StarredCourse",
    "Block",
]



from .backup_completion_status import BackupCompletionStatus
from .copy_data import CopyData, CopyDataResponse

from .base import BaseBackup

__all__ = [
    'BackupCompletionStatus',
    'CopyData',
    'CopyDataResponse',
    'BaseBackup',
]



from moodle import BaseMoodle
from moodle.utils.decorator import lazy
from . import BaseGuest, BaseManual, BaseSelf


class Enrol(BaseMoodle):
    @property  # type: ignore
    @lazy
    def guest(self) -> BaseGuest:
        return BaseGuest(self.moodle)

    @property  # type: ignore
    @lazy
    def manual(self) -> BaseManual:
        return BaseManual(self.moodle)

    @property  # type: ignore
    @lazy
    def self(self) -> BaseSelf:
        return BaseSelf(self.moodle)



from moodle.attr import dataclass


@dataclass
class CopyData:
    """Copy data

    Args:
        backupid (str): Backup id
        restoreid (str): Restore id
        operation (str): Operation type
    """
    backupid: str
    restoreid: str
    operation: str


@dataclass
class CopyDataResponse:
    """Copy completion status

    Args:
        status (int): Copy Status
        progress (float): Copy progress
        backupid (str): Copy id
        operation (str): Operation type
    """
    status: int
    progress: float
    backupid: str
    operation: str



from typing import List, Optional

from moodle.attr import dataclass, fields
from . import UserCustomField, UserPreference


@dataclass
class User:
    """User

    Args:
        id (int): ID of the user
        username (Optional[str]): The username
        firstname (Optional[str]): The first name(s) of the user
        lastname (Optional[str]): The family name of the user
        fullname (str): The fullname of the user
        email (Optional[str]): An email address - allow email as root@localhost
        address (Optional[str]): Postal address
        phone1 (Optional[str]): Phone 1
        phone2 (Optional[str]): Phone 2
        icq (Optional[str]): icq number
        skype (Optional[str]): skype id
        yahoo (Optional[str]): yahoo id
        aim (Optional[str]): aim id
        msn (Optional[str]): msn number
        department (Optional[str]): department
        institution (Optional[str]): institution
        idnumber (Optional[str]): An arbitrary ID code number perhaps from the institution
        interests (Optional[str]): user interests (separated by commas)
        firstaccess (Optional[int]): first access to the site (0 if never)
        lastaccess (Optional[int]): last access to the site (0 if never)
        auth (Optional[str]): Auth plugins include manual, ldap, etc
        suspended (Optional[int]): Suspend user account, either false to enable user login or true to disable it
        confirmed (Optional[int]): Active user: 1 if confirmed, 0 otherwise
        lang (Optional[str]): Language code such as "en", must exist on server
        calendartype (Optional[str]): Calendar type such as "gregorian", must exist on server
        theme (Optional[str]): Theme name such as "standard", must exist on server
        timezone (Optional[str]): Timezone code such as Australia/Perth, or 99 for default
        mailformat (Optional[int]): Mail format code is 0 for plain text, 1 for HTML etc
        description (Optional[str]): User profile description
        descriptionformat (Optional[int]): int format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        city (Optional[str]): Home city of the user
        url (Optional[str]): URL of the user
        country (Optional[str]): Home country code of the user, such as AU or CZ
        profileimageurlsmall (str): User image profile URL - small version
        profileimageurl (str): User image profile URL - big version
        customfields (List[UserCustomField]): User custom fields (also known as user profile fields)
        preferences (List[UserPreference]): Users preferences
    """
    id: int
    username: Optional[str]
    firstname: Optional[str]
    lastname: Optional[str]
    fullname: str
    email: Optional[str]
    address: Optional[str]
    phone1: Optional[str]
    phone2: Optional[str]
    icq: Optional[str]
    skype: Optional[str]
    yahoo: Optional[str]
    aim: Optional[str]
    msn: Optional[str]
    department: Optional[str]
    institution: Optional[str]
    idnumber: Optional[str]
    interests: Optional[str]
    firstaccess: Optional[int]
    lastaccess: Optional[int]
    auth: Optional[str]
    suspended: Optional[int]
    confirmed: Optional[int]
    lang: Optional[str]
    calendartype: Optional[str]
    theme: Optional[str]
    timezone: Optional[str]
    mailformat: Optional[int]
    description: Optional[str]
    descriptionformat: Optional[int]
    city: Optional[str]
    url: Optional[str]
    country: Optional[str]
    profileimageurlsmall: str
    profileimageurl: str
    customfields: List[UserCustomField] = fields(UserCustomField)
    preferences: List[UserPreference] = fields(UserPreference)



from typing import List, Optional

from moodle import MoodleWarning, ResponsesFactory
from moodle.attr import dataclass, fields


@dataclass
class File:
    """File
    Args:
        filename (Optional[str]): File name.
        filepath (Optional[str]): File path.
        filesize (Optional[int]): File size.
        fileurl (Optional[str]): Downloadable file url.
        timemodified (Optional[int]): Time modified.
        mimetype (Optional[str]): File mime type.
        isexternalfile (Optional[int]): Whether is an external file.
        repositorytype (Optional[str]): The repository type for external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class Url:
    """Url
    Args:
        id (int): Module id
        coursemodule (int): Course module id
        course (int): Course id
        name (str): URL name
        intro (str): Summary
        introformat (int): Default untuk "1" intro format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        introfiles (List[File]): Files in the introduction text
        externalurl (str): External URL
        display (int): How to display the url
        displayoptions (str): Display options (width, height)
        parameters (str): Parameters to append to the URL
        timemodified (int): Last time the url was modified
        section (int): Course section id
        visible (int): Module visibility
        groupmode (int): Group mode
        groupingid (int): Grouping id
    """
    id: int
    coursemodule: int
    course: int
    name: str
    intro: str
    introformat: int
    externalurl: str
    display: int
    displayoptions: str
    parameters: str
    timemodified: int
    section: int
    visible: int
    groupmode: int
    groupingid: int
    introfiles: List[File] = fields(File)


@dataclass
class Urls(ResponsesFactory[Url]):
    """List of Urls
    """
    urls: List[Url] = fields(Url)
    warnings: List[MoodleWarning] = fields(MoodleWarning)



from .signup_setting import SignupSetting
from .signup_user_response import SignupUserResponse
from .user_custom_field import UserCustomField
from .base import BaseEmail

__all__ = [
    'SignupSetting', 'SignupUserResponse', 'UserCustomField', 'BaseEmail'
]



from moodle.attr import dataclass


@dataclass
class UserGroup:
    """User Group

    Args:
        id (int): group id
        name (str): group name
        description (str): group description
        descriptionformat (int): description format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
    """
    id: int
    name: str
    description: str
    descriptionformat: int



from moodle.base.general import GeneralStatus


class EnrolUserResponse(GeneralStatus):
    """Enrol User Response

    Args:
        status (int): status: true if the user is enrolled, false otherwise
        warnings (List[MoodleWarning]): list of warnings
    """
    pass



from typing import List, Optional

from moodle import MoodleWarning, ResponsesFactory
from moodle.attr import dataclass, fields
from . import SiteNote, CourseNote, PersonalNote


@dataclass
class CourseNotes(ResponsesFactory[CourseNote]):
    """Course Notes
    Constructor arguments:
    params: sitenotes (List[SiteNote]): site notes
    params: coursenotes (List[CourseNote]): couse notes
    params: personalnotes (List[PersonalNote]): personal notes
    params: canmanagesystemnotes (Optional[int]): Whether the user can manage notes at system level.
    params: canmanagecoursenotes (Optional[int]): Whether the user can manage notes at the given course.
    params: warnings (List[Warning]): list of warnings
    """
    sitenotes: List[SiteNote] = fields(SiteNote)
    coursenotes: List[CourseNote] = fields(CourseNote)
    personalnotes: List[PersonalNote] = fields(PersonalNote)
    canmanagesystemnotes: Optional[int] = None
    canmanagecoursenotes: Optional[int] = None
    warnings: List[MoodleWarning] = fields(MoodleWarning)

    @property
    def items(self) -> List[CourseNote]:
        return self.coursenotes



from .version import __version__  # NOQA
from .base.moodle_object import MoodleObject
from .base.name_value import BaseNameValue
from .base.base_moodle import BaseMoodle
from .base.warning import MoodleWarning
from .base.responses import ResponsesFactory
from .base.preference import (
    MessagePreference,
    NotificationPreference,
    UserPreference,
)

from .exception import MoodleException
from .utils.typing import Array

from .tool import Tool
from .auth import Auth
from .block import Block
from .core import Core
from .enrol import Enrol
from .mod import Mod

from .mdl import Mdl
from .moodle import Moodle

__all__ = [
    'MoodleObject',
    'BaseNameValue',
    'BaseMoodle',
    'MoodleWarning',
    'ResponsesFactory',
    'MessagePreference',
    'NotificationPreference',
    'UserPreference',
    'MoodleException',
    'Array',
    'Auth',
    'Core',
    'Block',
    'Enrol',
    'Mod',
    'Tool',
    'Mdl',
    'Moodle',
]



from typing import List

from moodle import BaseMoodle
from . import Folders, View


class BaseFolder(BaseMoodle):
    def get_folders_by_courses(self, courseids: List[int]) -> Folders:
        """Returns a list of folders in a provided list of courses, if no list is provided all folders that the user can view will be returned.
        Please note that this WS is not returning the folder contents.

        Args:
            courseids (List[int]): Array of course ids

        Returns:
            Folders: List of Folder
        """
        res = self.moodle.post('mod_folder_get_folders_by_courses')
        return self._tr(Folders, **res)

    def view_folder(self, folderid: int) -> View:
        """Simulate the view.php web interface folder: trigger events, completion, etc...

        Args:
            folderid (int): folder instance id

        Returns:
            View: Response
        """
        res = self.moodle.post('mod_folder_view_folder')
        return self._tr(View, **res)



from moodle.attr import dataclass
from typing import List
from moodle import MoodleWarning


@dataclass
class ViewCourse:
    status: int  # status: true if success
    warnings: List[MoodleWarning]  # list of warnings



from moodle import BaseMoodle
from moodle.utils.decorator import lazy
from . import (
    BaseAuth,
    BaseBackup,
    BaseBadges,
    BaseBlock,
    BaseBlog,
    BaseCalendar,
    BaseCohort,
    BaseComment,
    BaseCompetency,
    BaseCompletion,
    BaseCourse,
    BaseCustomfield,
    BaseEnrol,
    BaseMessage,
    BaseNotes,
    BaseUser,
    BaseWebservice,
)


class Core(BaseMoodle):
    @property  # type: ignore
    @lazy
    def auth(self) -> BaseAuth:
        return BaseAuth(self.moodle)

    @property  # type: ignore
    @lazy
    def backup(self) -> BaseBackup:
        return BaseBackup(self.moodle)

    @property  # type: ignore
    @lazy
    def bagdes(self) -> BaseBadges:
        return BaseBadges(self.moodle)

    @property  # type: ignore
    @lazy
    def block(self) -> BaseBlock:
        return BaseBlock(self.moodle)

    @property  # type: ignore
    @lazy
    def blog(self) -> BaseBlog:
        return BaseBlog(self.moodle)

    @property  # type: ignore
    @lazy
    def calendar(self) -> BaseCalendar:
        return BaseCalendar(self.moodle)

    @property  # type: ignore
    @lazy
    def cohort(self) -> BaseCohort:
        return BaseCohort(self.moodle)

    @property  # type: ignore
    @lazy
    def comment(self) -> BaseComment:
        return BaseComment(self.moodle)

    @property  # type: ignore
    @lazy
    def competency(self) -> BaseCompetency:
        return BaseCompetency(self.moodle)

    @property  # type: ignore
    @lazy
    def completion(self) -> BaseCompletion:
        return BaseCompletion(self.moodle)

    @property  # type: ignore
    @lazy
    def course(self) -> BaseCourse:
        return BaseCourse(self.moodle)

    @property  # type: ignore
    @lazy
    def customfield(self) -> BaseCustomfield:
        return BaseCustomfield(self.moodle)

    @property  # type: ignore
    @lazy
    def enrol(self) -> BaseEnrol:
        return BaseEnrol(self.moodle)

    @property  # type: ignore
    @lazy
    def message(self) -> BaseMessage:
        return BaseMessage(self.moodle)

    @property  # type: ignore
    @lazy
    def notes(self) -> BaseNotes:
        return BaseNotes(self.moodle)

    @property  # type: ignore
    @lazy
    def user(self) -> BaseUser:
        return BaseUser(self.moodle)

    @property  # type: ignore
    @lazy
    def webservice(self) -> BaseWebservice:
        return BaseWebservice(self.moodle)



from moodle.attr import dataclass
from typing import List

from . import EnrolledUser


@dataclass
class CapabilityEnrolledUser:
    """Capability with Enrolled Users

    Args:
        courseid (int): Course ID number in the Moodle course table
        capability (str): Capability name
        users (List[EnrolledUser]): List of users that are enrolled in the course and have the specified capability
    """
    courseid: int
    capability: str
    users: List[EnrolledUser]



from typing import List

from moodle import MoodleWarning
from moodle.attr import dataclass, fields


@dataclass
class Key:
    """Autologin key
    params: key (str): Auto-login key for a single usage with time expiration.
    params: autologinurl (str): Auto-login URL.
    params: warnings (List[Warning]): list of warnings
    """
    key: str
    autologinurl: str
    warnings: List[MoodleWarning] = fields(MoodleWarning)

    def __str__(self) -> str:
        return self.key



from datetime import datetime
from typing import List, Union

from moodle import BaseMoodle


class BaseChat(BaseMoodle):
    def get_chat_latest_messages(self,
                                 chatsid: str,
                                 chatlasttime: Union[int, datetime] = 0):
        data = self.moodle.post(
            'mod_chat_get_chat_latest_messages',
            chatsid=chatsid,
            chatlasttime=chatlasttime,
        )
        return data

    def get_chat_users(self, chatsid: str):
        data = self.moodle.post(
            'mod_chat_get_chat_users',
            chatsid=chatsid,
        )
        return data

    def get_chats_by_courses(self, courseids: List[int]):
        data = self.moodle.post(
            'mod_chat_get_chats_by_courses',
            courseids=courseids,
        )
        return data

    def get_session_messages(self,
                             chatid: int,
                             sessionstart: Union[int, datetime],
                             sessionend: Union[int, datetime],
                             groupid: int = 0):
        data = self.moodle.post(
            'mod_chat_get_session_messages',
            chatid=chatid,
            sessionstart=sessionstart,
            sessionend=sessionend,
            groupid=groupid,
        )
        return data

    def get_sessions(self, chatid: int, groupid: int = 0, showall: int = None):
        data = self.moodle.post(
            'mod_chat_get_sessions',
            chatid=chatid,
            groupid=groupid,
            showall=showall or '',
        )
        return data

    def login_user(self, chatid: int, groupid: int = 0):
        data = self.moodle.post(
            'mod_chat_login_user',
            chatid=chatid,
            groupid=groupid,
        )
        return data

    def send_chat_message(self,
                          chatsid: str,
                          messagetext: str,
                          beepid: str = ''):
        data = self.moodle.post(
            'mod_chat_send_chat_message',
            chatsid=chatsid,
            messagetext=messagetext,
            beepid=beepid,
        )
        return data

    def view_chat(self, chatid: int):
        data = self.moodle.post(
            'mod_chat_view_chat',
            chatid=chatid,
        )
        return data



from moodle.attr import dataclass


@dataclass
class RecentlyAccessedItem:
    """Recently Accessed Item

    Args:
        id (int): id
        courseid (int): courseid
        cmid (int): cmid
        userid (int): userid
        modname (str): modname
        name (str): name
        coursename (str): coursename
        timeaccess (int): timeaccess
        viewurl (str): viewurl
        courseviewurl (str): courseviewurl
        icon (str): icon
    """
    id: int
    courseid: int
    cmid: int
    userid: int
    modname: str
    name: str
    coursename: str
    timeaccess: int
    viewurl: str
    courseviewurl: str
    icon: str



from typing import Optional, List

from moodle.attr import dataclass, fields
from moodle import MoodleWarning, ResponsesFactory


@dataclass
class BagdeEndorsement:
    """Bagde Endorsement
    Constructor arguments:
        id (int): Endorsement id
        badgeid (int): Badge id
        issuername (str): Endorsement issuer name
        issuerurl (str): Endorsement issuer URL
        issueremail (str): Endorsement issuer email
        claimid (str): Claim URL
        claimcomment (str): Claim comment
        dateissued (int): Date issued
    """
    id: int
    badgeid: int
    issuername: str
    issuerurl: str
    issueremail: str
    claimid: str
    claimcomment: str
    dateissued: int


@dataclass
class BagdeAlignment:
    """Bagde Alignment
    Constructor arguments:
        id (Optional[int]): Alignment id
        badgeid (Optional[int]): Badge id
        targetName (Optional[str]): Target name
        targetUrl (Optional[str]): Target URL
        targetDescription (Optional[str]): Target description
        targetFramework (Optional[str]): Target framework
        targetCode (Optional[str]): Target code
    """
    id: Optional[int]
    badgeid: Optional[int]
    targetName: Optional[str]
    targetUrl: Optional[str]
    targetDescription: Optional[str]
    targetFramework: Optional[str]
    targetCode: Optional[str]


@dataclass
class RelatedBadge:
    """Related Badge
    Constructor arguments:
        id (int): Badge id
        name (str): Badge name
        version (Optional[str]): Version
        language (Optional[str]): Language
        type (Optional[int]): Type
    """
    id: int
    name: str
    version: Optional[str]
    language: Optional[str]
    type: Optional[int]


@dataclass
class Badge:
    """Badge
    Constructor arguments:
        id (Optional[int]): Badge id
        name (str): Badge name
        description (str): Badge description
        timecreated (Optional[int]): Time created
        timemodified (Optional[int]): Time modified
        usercreated (Optional[int]): User created
        usermodified (Optional[int]): User modified
        issuername (str): Issuer name
        issuerurl (str): Issuer URL
        issuercontact (str): Issuer contact
        expiredate (Optional[int]): Expire date
        expireperiod (Optional[int]): Expire period
        type (Optional[int]): Type
        courseid (Optional[int]): Course id
        message (Optional[str]): Message
        messagesubject (Optional[str]): Message subject
        attachment (Optional[int]): Attachment
        notification (Optional[int]): Whether to notify when badge is awarded
        nextcron (Optional[int]): Next cron
        status (Optional[int]): Status
        issuedid (Optional[int]): Issued id
        uniquehash (str): Unique hash
        dateissued (int): Date issued
        dateexpire (Optional[int]): Date expire
        visible (Optional[int]): Visible
        email (Optional[str]): User email
        version (Optional[str]): Version
        language (Optional[str]): Language
        imageauthorname (Optional[str]): Name of the image author
        imageauthoremail (Optional[str]): Email of the image author
        imageauthorurl (Optional[str]): URL of the image author
        imagecaption (Optional[str]): Caption of the image
        badgeurl (str): Badge URL
        endorsement (Optional[BagdeEndorsement]): Badge endorsement
        alignment (List[BagdeAlignment]): Badge alignments
        relatedbadges (List[RelatedBadge]): Related badges
    """
    id: Optional[int]
    name: str
    description: str
    timecreated: Optional[int]
    timemodified: Optional[int]
    usercreated: Optional[int]
    usermodified: Optional[int]
    issuername: str
    issuerurl: str
    issuercontact: str
    expiredate: Optional[int]
    expireperiod: Optional[int]
    type: Optional[int]
    courseid: Optional[int]
    message: Optional[str]
    messagesubject: Optional[str]
    attachment: Optional[int]
    notification: Optional[int]
    nextcron: Optional[int]
    status: Optional[int]
    issuedid: Optional[int]
    uniquehash: str
    dateissued: int
    dateexpire: Optional[int]
    visible: Optional[int]
    email: Optional[str]
    version: Optional[str]
    language: Optional[str]
    imageauthorname: Optional[str]
    imageauthoremail: Optional[str]
    imageauthorurl: Optional[str]
    imagecaption: Optional[str]
    badgeurl: str
    endorsement: Optional[BagdeEndorsement]
    alignment: List[BagdeAlignment] = fields(BagdeAlignment)
    relatedbadges: List[RelatedBadge] = fields(RelatedBadge)


@dataclass
class BadgeResponse(ResponsesFactory[Badge]):
    """BadgeResponse
    Constructor arguments:
        badges (List[Badge]): list of Badge
        warnings (List[Warning]): list of warnings
    """
    badges: List[Badge] = fields(Badge)
    warnings: List[MoodleWarning] = fields(MoodleWarning)

    @property
    def items(self) -> List[Badge]:
        return self.badges



import logging
import requests
from requests import Session
from requests.exceptions import RequestException
from typing import Any, Dict

from moodle import __version__
from moodle import MoodleException
from moodle import MoodleWarning

from moodle.exception import EmptyResponseException, InvalidCredentialException, NetworkMoodleException
from moodle.utils.helper import make_params, to_dict


class Mdl:
    session: Session = Session()
    token: str = ''
    url: str = ''
    __headers__: Dict[str, str] = {'User-Agent': f"moodlepy-{__version__}"}

    def __init__(self, url: str, token: str, user_agent: str = None):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.url = url
        self.token = token
        if user_agent:
            self.__headers__['User-Agent'] = user_agent
        self.session.headers.update(self.__headers__)

    def get(self, wsfunction: str, moodlewsrestformat='json', **kwargs) -> Any:
        params = make_params(self.token, wsfunction, moodlewsrestformat)
        params.update(to_dict(kwargs))
        res = self.session.get(self.url, params=params)
        if res.ok and moodlewsrestformat == 'json':
            data = res.json()
            return self.process_response(data)
        return res.text

    def post(self,
             wsfunction: str,
             moodlewsrestformat='json',
             **kwargs: Any) -> Any:
        params = make_params(self.token, wsfunction, moodlewsrestformat)
        try:
            res = self.session.post(
                self.url,
                data=to_dict(kwargs),
                params=params,
            )
        except RequestException as e:
            raise NetworkMoodleException(e)
        if not res.ok or not res.text:
            raise EmptyResponseException()
        if res.ok and moodlewsrestformat == 'json':
            data = res.json()
            return self.process_response(data)
        return res.text

    def process_response(self, data: Any) -> Any:
        if type(data) == dict:
            if 'warnings' in data and data['warnings']:
                warning = MoodleWarning(**data['warnings'])  # type: ignore
                self.logger.warning(str(warning))
            if 'exception' in data or 'errorcode' in data:
                raise MoodleException(**data)  # type: ignore
        return data

    @classmethod
    def login(cls,
              domain: str,
              username: str,
              password: str,
              service: str = 'moodle_mobile_app',
              loginurl: str = '/login/token.php',
              web_service_url: str = '/webservice/rest/server.php'):
        """Get a Moodle instance by using username & password auth

        Args:
            domain (str): The domain of moodle app
            username (str): Username
            password (str): Password
            service (str, optional): Service name. Defaults to 'moodle_mobile_app'.
            loginurl (str, optional): Url to get token from. Defaults to '/login/token.php'.
            web_service_url (str, optional): Moodle Web Service endpoint. Defaults to '/webservice/rest/server.php'.

        Raises:
            InvalidCredentialException: Wrong credential

        Returns:
            Moodle: Returns an instance of Moodle
        """
        data = cls.get_tokens(domain, username, password, service, loginurl)
        if 'token' not in data:
            raise InvalidCredentialException()
        return cls(url=web_service_url if web_service_url.startswith(domain)
                   else domain + web_service_url,
                   token=data['token'])

    @classmethod
    def get_tokens(
        cls,
        domain: str,
        username: str,
        password: str,
        service: str = 'moodle_mobile_app',
        loginurl: str = '/login/token.php',
    ) -> dict:
        domain = domain.lstrip('/')
        params = {
            'username': username,
            'password': password,
            'service': service
        }
        url = loginurl if loginurl.startswith(domain) else domain + loginurl
        res = requests.get(url, params=params)
        return res.json() if res.ok else {}



import logging
from functools import wraps
from typing import Callable, Type
from moodle import MoodleObject


def meta(wsfunction: str, moodlewsrestformat: str) -> Callable:
    def decorator(cls: Type[MoodleObject]) -> Type[MoodleObject]:
        setattr(cls, 'wsfunction', wsfunction)
        setattr(cls, 'moodlewsrestformat', moodlewsrestformat)
        return cls

    return decorator


def deprecated(func: Callable):
    """Specify the method is deprecated

    Args:
        func (Callable): Wsfunction of moodlepy

    Returns:
        Callable: Decorated func
    """
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        logging.warn(
            f'{func.__name__} **DEPRECATED** Please do not call this function any more',
            DeprecationWarning)
        return func(*args, **kwargs)

    return wrapper


def lazy(func):
    '''Decorator that makes a property lazy-evaluated.
    Source : https://stevenloria.com/lazy-properties/
    '''
    attr_name = '_lazy_' + func.__name__

    @wraps(func)
    def wrapper(self):
        if not hasattr(self, attr_name):
            setattr(self, attr_name, func(self))
        return getattr(self, attr_name)

    return wrapper



from typing import Any

from moodle.mdl import Mdl

from moodle.auth import Auth
from moodle.block import Block
from moodle.core import Core
from moodle.enrol import Enrol
from moodle.mod import Mod
from moodle.tool import Tool

from moodle.utils.decorator import lazy


class Moodle(Mdl):
    def __init__(self, url: str, token: str):
        super(Moodle, self).__init__(url, token)

    def __call__(self,
                 wsfunction: str,
                 moodlewsrestformat='json',
                 **kwargs) -> Any:
        return self.post(wsfunction, moodlewsrestformat, **kwargs)

    @property  # type: ignore
    @lazy
    def auth(self) -> Auth:
        return Auth(self)

    @property  # type: ignore
    @lazy
    def block(self) -> Block:
        return Block(self)

    @property  # type: ignore
    @lazy
    def core(self) -> Core:
        return Core(self)

    @property  # type: ignore
    @lazy
    def enrol(self) -> Enrol:
        return Enrol(self)

    @property  # type: ignore
    @lazy
    def mod(self) -> Mod:
        return Mod(self)

    @property  # type: ignore
    @lazy
    def tool(self) -> Tool:
        return Tool(self)



from moodle import BaseMoodle
from . import InstanceInfo
from .enrol_user_response import EnrolUserResponse


class BaseSelf(BaseMoodle):
    def enrol_user(self,
                   courseid: int,
                   password: str,
                   instanceid: int = 0) -> EnrolUserResponse:
        """Self enrol the current user in the given course.

        Args:
            courseid (int): Id of the course
            password (str): Enrolment key
            instanceid (int, optional): Instance id of self enrolment plugin. Defaults to 0.

        Returns:
            EnrolUserResponse: Response
        """
        data = self.moodle.post(
            'enrol_self_enrol_user',
            courseid=courseid,
            password=password,
            instanceid=instanceid,
        )
        return self._tr(EnrolUserResponse, **data)

    def get_instance_info(self, instanceid: int) -> InstanceInfo:
        """self enrolment instance information.

        Args:
            instanceid (int): instance id of self enrolment plugin.

        Returns:
            InstanceInfo: Instance info
        """
        data = self.moodle.post(
            'enrol_self_get_instance_info',
            instanceid=instanceid,
        )
        return self._tr(InstanceInfo, **data)



from typing import List

from moodle import MoodleWarning
from moodle.attr import dataclass, fields


@dataclass
class StatusCompletion:
    """Status Completion
    Args:
        status (int): status, true if success
        warnings (List[Warning]): list of warnings
    """
    status: int
    warnings: List[MoodleWarning] = fields(MoodleWarning)



from moodle.attr import dataclass


@dataclass
class TableRowData:
    """Table row data.

    Args:
        filesize (str): Backup file size
        fileurl (str): Backup file URL
        restoreurl (str): Backup restore URL
    """
    filesize: str
    fileurl: str
    restoreurl: str



from typing import List, Optional

from moodle import MoodleWarning, ResponsesFactory
from moodle.attr import dataclass, fields


@dataclass
class File:
    """File

    Args:
        filename (Optional[str]): File name.
        filepath (Optional[str]): File path.
        filesize (Optional[int]): File size.
        fileurl (Optional[str]): Downloadable file url.
        timemodified (Optional[int]): Time modified.
        mimetype (Optional[str]): File mime type.
        isexternalfile (Optional[int]): Whether is an external file.
        repositorytype (Optional[str]): The repository type for external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class Page:
    """Page

    Args:
        id (int): Module id
        coursemodule (int): Course module id
        course (int): Course id
        name (str): Page name
        intro (str): Summary
        introformat (int): Default to "1" intro format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        introfiles (List[File]): Files in the introduction text
        content (str): Page content
        contentformat (int): Default to "1" content format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        contentfiles (List[File]): Files in the content
        legacyfiles (int): Legacy files flag
        legacyfileslast (int): Legacy files last control flag
        display (int): How to display the page
        displayoptions (str): Display options (width, height)
        revision (int): Incremented when after each file changes, to avoid cache
        timemodified (int): Last time the page was modified
        section (int): Course section id
        visible (int): Module visibility
        groupmode (int): Group mode
        groupingid (int): Grouping id
    """
    id: int
    coursemodule: int
    course: int
    name: str
    intro: str
    introformat: int
    introfiles: List[File]
    content: str
    contentformat: int
    contentfiles: List[File]
    legacyfiles: int
    legacyfileslast: int
    display: int
    displayoptions: str
    revision: int
    timemodified: int
    section: int
    visible: int
    groupmode: int
    groupingid: int


@dataclass
class PagesResponse(ResponsesFactory[Page]):
    """

    Args:
        pages (List[Page]): list of Page
        warnings (List[MoodleWarning]): list of MoodleWarning
    """
    pages: List[Page] = fields(Page)
    warnings: List[MoodleWarning] = fields(MoodleWarning)

    @property
    def items(self) -> List[Page]:
        return self.pages



from moodle.attr import dataclass
from typing import List


@dataclass
class CourseCapability:
    """Course Capability

    Args:
        courseid (int): Course ID number in the Moodle course table
        capabilities (List[str]): Capability name, such as mod/forum:viewdiscussion
    """
    courseid: int
    capabilities: List[str]



from moodle.attr import dataclass
from typing import List
from moodle import MoodleWarning


@dataclass
class RequestPasswordResetResponse:
    """RequestPasswordResetResponse

    Args:
        status (str): The returned status of the process:
                        dataerror: Error in the sent data (username or email). More information in warnings field.
                        emailpasswordconfirmmaybesent: Email sent or not (depends on user found in database).
                        emailpasswordconfirmnotsent: Failure, user not found.
                        emailpasswordconfirmnoemail: Failure, email not found.
                        emailalreadysent: Email already sent.
                        emailpasswordconfirmsent: User pending confirmation.
                        emailresetconfirmsent: Email sent.
        notice (str): Important information for the user about the process.
        warnings (List[Warning]): list of warnings
    """
    status: str
    notice: str
    warnings: List[MoodleWarning]



from moodle.base.general import GeneralTypeValue


class UserPreference(GeneralTypeValue):
    """User preferences

    Args:
        type (str): The name of the preference
        value (str): The value of the preference
    """
    pass



from moodle import BaseMoodle
from moodle.utils.decorator import lazy
from . import BaseMobile


class Tool(BaseMoodle):
    @property  # type: ignore
    @lazy
    def mobile(self) -> BaseMobile:
        return BaseMobile(self.moodle)



from moodle.attr import dataclass
from typing import List

from moodle import MoodleWarning


@dataclass
class GeneralSuccess:
    """GeneralSuccess
    Args:
        success (int): True or False, nor 1 or 0
        warnings (List[Warning]): list of warnings
    """
    success: int
    warnings: List[MoodleWarning]

    def __bool__(self) -> bool:
        if isinstance(self.success, int):
            return self.success == 1
        elif isinstance(self.success, str):
            return self.success == '1' or self.success == 'true'
        return bool(self.success)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['moodle',
 'moodle.auth',
 'moodle.auth.email',
 'moodle.base',
 'moodle.base.general',
 'moodle.base.preference',
 'moodle.block',
 'moodle.core',
 'moodle.core.auth',
 'moodle.core.backup',
 'moodle.core.badges',
 'moodle.core.block',
 'moodle.core.blog',
 'moodle.core.calendar',
 'moodle.core.cohort',
 'moodle.core.comment',
 'moodle.core.competency',
 'moodle.core.completion',
 'moodle.core.course',
 'moodle.core.customfield',
 'moodle.core.enrol',
 'moodle.core.files',
 'moodle.core.message',
 'moodle.core.notes',
 'moodle.core.user',
 'moodle.core.webservice',
 'moodle.enrol',
 'moodle.enrol.guest',
 'moodle.enrol.manual',
 'moodle.enrol.self',
 'moodle.mod',
 'moodle.mod.assign',
 'moodle.mod.chat',
 'moodle.mod.data',
 'moodle.mod.folder',
 'moodle.mod.forum',
 'moodle.mod.lesson',
 'moodle.mod.page',
 'moodle.mod.resource',
 'moodle.mod.url',
 'moodle.mod.workshop',
 'moodle.tool',
 'moodle.tool.mobile',
 'moodle.utils']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=20.3.0,<21.0.0', 'requests>=2.24.0,<3.0.0']

entry_points = \
{'console_scripts': ['moodle = moodle.__main__:main']}

setup_kwargs = {
    'name': 'moodlepy',
    'version': '0.22.0',
    'description': 'Python wrapper for moodle web service.',
    'long_description': "# moodlepy\n\n[![PyPi Package Version](https://img.shields.io/pypi/v/moodlepy)](https://pypi.org/project/moodlepy/)\n[![Supported Python versions](https://img.shields.io/pypi/pyversions/moodlepy](https://pypi.org/project/moodlepy/)\n[![LICENSE](https://img.shields.io/github/license/hexatester/moodlepy)](https://github.com/hexatester/moodlepy/blob/master/LICENSE)\n[![Tests](https://github.com/hexatester/moodlepy/workflows/Tests/badge.svg)](https://github.com/hexatester/moodlepy/actions?query=workflow%3ATests)\n[![codecov](https://codecov.io/gh/hexatester/moodlepy/branch/master/graph/badge.svg)](https://codecov.io/gh/hexatester/moodlepy)\n[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/pre-commit/pre-commit)\n[![Mypy](https://img.shields.io/badge/Mypy-enabled-brightgreen)](https://github.com/python/mypy)\n\nPython wrapper for moodle web service.\n\n## Introduction\n\nThis library provide a pure Python interface for [Moodle Web Service](https://docs.moodle.org/dev/Web_services). It's compatible with Python versions 3.6+\n\n## Installing\n\nYou can install or upgrade moodlepy with:\n\n```bash\npip install moodlepy --upgrade\n```\n\nOr you can install from source with:\n\n```bash\ngit clone https://github.com/hexatester/moodlepy\ncd moodlepy\npython setup.py install\n```\n\n## Usage\n\nExample usage\n\n```python\nfrom moodle import Moodle\nurl = 'https://my.domain/webservice/rest/server.php'\ntoken = 'super secret token'\nmoodle = Moodle(url, token)\nraw_site_info = moodle('core_webservice_get_site_info')\nsite_info = moodle.core.webservice.get_site_info()  # return typed site_info\n\nprint(raw_site_info)\nprint(site_info)\n\n# or\nfrom moodle import Mdl\nfrom moodle.core.webservice import BaseWebservice\n\nmoodle = Mdl(url, token)\nwebservice = BaseWebservice(moodle)\nsite_info2 = webservice.get_site_info()\n\nassert site_info == site_info2\n```\n\nIn the future all [Web service functions](https://docs.moodle.org/dev/Web_service_API_functions) will be covered by moodlepy\n\n# Moodle Web Service support\n\n❗️ Not all types and methods are supported, since moodlepy is not yet released. (A = Added.)\n\n| Area                 | Functions | Types | Tests | Status |\n| -------------------- | --------- | ----- | ----- | ------ |\n| auth_email           | A         | A     |       |        |\n| block                | A         | A     |       |        |\n| core_auth            | A         | A     |       |        |\n| core_backup          | A         | A     |       |        |\n| core_badge           | A         | A     | A     |        |\n| core_blog            | A         | A     | A     |        |\n| core_calendar        | A         | A     |       |        |\n| core_cohort          |           |       |       |        |\n| core_comment         |           |       |       |        |\n| core_competency      |           |       |       |        |\n| core_completion      | A         | A     |       |        |\n| core_course          |           |       |       |        |\n| core_customfield     | A         | A     |       |        |\n| core_enrol           |           |       |       |        |\n| core_fetch           |           |       |       |        |\n| core_files           |           |       |       |        |\n| core_filters         |           |       |       |        |\n| core_form            |           |       |       |        |\n| core_get             |           |       |       |        |\n| core_grade           |           |       |       |        |\n| core_grades          |           |       |       |        |\n| core_grading         |           |       |       |        |\n| core_group           |           |       |       |        |\n| core_h5p             |           |       |       |        |\n| core_message         | A         |       |       |        |\n| core_notes           | A         | A     |       |        |\n| core_output          |           |       |       |        |\n| core_question        |           |       |       |        |\n| core_rating          |           |       |       |        |\n| core_role            |           |       |       |        |\n| core_search          |           |       |       |        |\n| core_session         |           |       |       |        |\n| core_tag             |           |       |       |        |\n| core_update          |           |       |       |        |\n| core_user            |           |       |       |        |\n| core_webservice      | A         | A     | A     | A      |\n| enrol_guest          | A         | A     |       |        |\n| enrol_manual         | A         | A     |       |        |\n| enrol_self           | A         | A     |       |        |\n| gradereport_overview |           |       |       |        |\n| gradereport_user     |           |       |       |        |\n| gradingform_guide    |           |       |       |        |\n| gradingform_rubric   |           |       |       |        |\n| local_mobile         |           |       |       |        |\n| message_airnotifier  |           |       |       |        |\n| message_popup        |           |       |       |        |\n| mod_assign           |           |       |       |        |\n| mod_book             |           |       |       |        |\n| mod_chat             | A         |       |       |        |\n| mod_choice           |           |       |       |        |\n| mod_data             | A         |       |       |        |\n| mod_feedback         |           |       |       |        |\n| mod_folder           | A         | A     |       |        |\n| mod_forum            |           |       |       |        |\n| mod_glossary         |           |       |       |        |\n| mod_imscp            |           |       |       |        |\n| mod_label            |           |       |       |        |\n| mod_lesson           |           |       |       |        |\n| mod_lti              |           |       |       |        |\n| mod_page             | A         | A     |       |        |\n| mod_quiz             |           |       |       |        |\n| mod_resource         | A         | A     |       |        |\n| mod_scorm            |           |       |       |        |\n| mod_survey           |           |       |       |        |\n| mod_url              | A         | A     |       |        |\n| mod_wiki             |           |       |       |        |\n| mod_workshop         | A         |       |       |        |\n| report_competency    |           |       |       |        |\n| report_insights      |           |       |       |        |\n| tool_analytics       |           |       |       |        |\n| tool_lp              |           |       |       |        |\n| tool_mobile          | A         | A     |       |        |\n| tool_templatelibrary |           |       |       |        |\n| tool_usertours       |           |       |       |        |\n| tool_xmldb           |           |       |       |        |\n",
    'author': 'hexatester',
    'author_email': 'habibrohman@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://hexatester.github.io/moodlepy/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



from moodle.attr import dataclass
from typing import List

from moodle import MoodleWarning, ResponsesFactory


@dataclass
class ParentPaths(ResponsesFactory[str]):
    """Parent Paths

    Args
        parentpaths (List[str]): Path to parent directory of the deleted files.
        warnings (List[Warning]): list of warnings
    """
    parentpaths: List[str]
    warnings: List[MoodleWarning]

    @property
    def items(self) -> List[str]:
        return self.parentpaths



from typing import List, Optional

from moodle.attr import dataclass, fields
from . import UserCustomField, UserPreference


@dataclass
class CreateUser:
    """Create User

    Args:
        createpassword (Optional[int]): True if password should be created and mailed to user.
        username (str): Username policy is defined in Moodle security config.
        auth (str): Default to "manual" #Auth plugins include manual, ldap, etc
        password (Optional[str]): Plain text password consisting of any characters
        firstname (str): The first name(s) of the user
        lastname (str): The family name of the user
        email (str): A valid and unique email address
        maildisplay (Optional[int]): Email display
        city (Optional[str]): Home city of the user
        country (Optional[str]): Home country code of the user, such as AU or CZ
        timezone (Optional[str]): Timezone code such as Australia/Perth, or 99 for default
        description (Optional[str]): User profile description, no HTML
        firstnamephonetic (Optional[str]): The first name(s) phonetically of the user
        lastnamephonetic (Optional[str]): The family name phonetically of the user
        middlename (Optional[str]): The middle name of the user
        alternatename (Optional[str]): The alternate name of the user
        interests (Optional[str]): User interests (separated by commas)
        url (Optional[str]): User web page
        icq (Optional[str]): ICQ number
        skype (Optional[str]): Skype ID
        aim (Optional[str]): AIM ID
        yahoo (Optional[str]): Yahoo ID
        msn (Optional[str]): MSN ID
        idnumber (str): Default to "" An arbitrary ID code number perhaps from the institution
        institution (Optional[str]): institution
        department (Optional[str]): department
        phone1 (Optional[str]): Phone 1
        phone2 (Optional[str]): Phone 2
        address (Optional[str]): Postal address
        lang (str): Default to "en" #Language code such as "en", must exist on server
        calendartype (str): Default to "gregorian" Calendar type such as "gregorian", must exist on server
        theme (Optional[str]): Theme name such as "standard", must exist on server
        mailformat (Optional[int]): Mail format code is 0 for plain text, 1 for HTML etc
        customfields (List[UserCustomField]): User custom fields (also known as user profil fields)
        preferences (List[UserPreference]): User preferences
    """
    createpassword: Optional[int]
    username: str
    auth: str
    password: Optional[str]
    firstname: str
    lastname: str
    email: str
    maildisplay: Optional[int]
    city: Optional[str]
    country: Optional[str]
    timezone: Optional[str]
    description: Optional[str]
    firstnamephonetic: Optional[str]
    lastnamephonetic: Optional[str]
    middlename: Optional[str]
    alternatename: Optional[str]
    interests: Optional[str]
    url: Optional[str]
    icq: Optional[str]
    skype: Optional[str]
    aim: Optional[str]
    yahoo: Optional[str]
    msn: Optional[str]
    idnumber: str
    institution: Optional[str]
    department: Optional[str]
    phone1: Optional[str]
    phone2: Optional[str]
    address: Optional[str]
    theme: Optional[str]
    mailformat: Optional[int]
    lang: str = 'en'
    calendartype: str = 'gregorian'
    customfields: List[UserCustomField] = fields(UserCustomField)
    preferences: List[UserPreference] = fields(UserPreference)

    @dataclass
    class Response:
        """Response for CreateUser

        Args:
            id (int): user id
            username (string): user name
        """
        id: int
        username: str



from moodle.attr import dataclass, fields
from typing import List
from . import GeneralKeyMessage


@dataclass
class GeneralResultError:
    """GeneralResultError
    Args:
        result (int): True if the user's enrolment was successfully updated
        errors (List[GeneralKeyMessage]): List of validation errors
    """
    result: int
    errors: List[GeneralKeyMessage] = fields(GeneralKeyMessage)

    def __bool__(self) -> bool:
        if isinstance(self.result, int):
            return self.result == 1
        elif isinstance(self.result, str):
            return self.result == '1' or self.result == 'true'
        return bool(self.result)



from typing import List

from moodle.attr import dataclass, fields
from . import (
    User,
    UserGroup,
    UserRole,
    EnrolledCourse,
)


@dataclass
class UserProfile(User):
    """User Profile

    Args:
        id (int): ID of the user
        username (Optional[str]): The username
        firstname (Optional[str]): The first name(s) of the user
        lastname (Optional[str]): The family name of the user
        fullname (str): The fullname of the user
        email (Optional[str]): An email address - allow email as root@localhost
        address (Optional[str]): Postal address
        phone1 (Optional[str]): Phone 1
        phone2 (Optional[str]): Phone 2
        icq (Optional[str]): icq number
        skype (Optional[str]): skype id
        yahoo (Optional[str]): yahoo id
        aim (Optional[str]): aim id
        msn (Optional[str]): msn number
        department (Optional[str]): department
        institution (Optional[str]): institution
        idnumber (Optional[str]): An arbitrary ID code number perhaps from the institution
        interests (Optional[str]): user interests (separated by commas)
        firstaccess (Optional[int]): first access to the site (0 if never)
        lastaccess (Optional[int]): last access to the site (0 if never)
        auth (Optional[str]): Auth plugins include manual, ldap, etc
        suspended (Optional[int]): Suspend user account, either false to enable user login or true to disable it
        confirmed (Optional[int]): Active user: 1 if confirmed, 0 otherwise
        lang (Optional[str]): Language code such as "en", must exist on server
        calendartype (Optional[str]): Calendar type such as "gregorian", must exist on server
        theme (Optional[str]): Theme name such as "standard", must exist on server
        timezone (Optional[str]): Timezone code such as Australia/Perth, or 99 for default
        mailformat (Optional[int]): Mail format code is 0 for plain text, 1 for HTML etc
        description (Optional[str]): User profile description
        descriptionformat (Optional[int]): int format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        city (Optional[str]): Home city of the user
        url (Optional[str]): URL of the user
        country (Optional[str]): Home country code of the user, such as AU or CZ
        profileimageurlsmall (str): User image profile URL - small version
        profileimageurl (str): User image profile URL - big version
        customfields (List[UserCustomField]): User custom fields (also known as user profile fields)
        preferences (List[UserPreference]): Users preferences
        groups (List[Group]): user groups
        roles (List[Role]): user roles
        enrolledcourses (List[EnrolledCourse]): Courses where the user is enrolled - limited by which courses the user is able to see
    """
    groups: List[UserGroup] = fields(UserGroup)
    roles: List[UserRole] = fields(UserRole)
    enrolledcourses: List[EnrolledCourse] = fields(EnrolledCourse)



from moodle import BaseMoodle
from . import ReloadTemplate


class BaseCustomfield(BaseMoodle):
    def create_category(self, component: str, area: str, itemid: int) -> int:
        """Creates a new category

        Args:
            component (str): component
            area (str): area
            itemid (int): itemid

        Returns:
            int: Id of the category
        """
        data = self.moodle.post(
            'core_customfield_create_category',
            component=component,
            area=area,
            itemid=itemid,
        )
        return data

    def delete_category(self, id: int) -> None:
        """Deletes a category

        Args:
            id (int): category ID to delete
        """
        self.moodle.post('core_customfield_delete_category', id=id)

    def delete_field(self, id: int) -> None:
        """Deletes an entry

        Args:
            id (int): Custom field ID to delete
        """
        self.moodle.post('core_customfield_delete_field', id=id)

    def move_category(self, id: int, beforeid: int = 0) -> None:
        """Drag and drop categories

        Args:
            id (int): Category ID to move
            beforeid (int, optional): Id of the category before which it needs to be moved. Defaults to 0.
        """
        self.moodle.post(
            'core_customfield_move_category',
            id=id,
            beforeid=beforeid,
        )

    def move_field(self, id: int, categoryid: int, beforeid: int = 0) -> None:
        """Drag and drop

        Args:
            id (int): Id of the field to move
            categoryid (int): New parent category id
            beforeid (int, optional): Id of the field before which it needs to be moved. Defaults to 0.
        """
        self.moodle.post(
            'core_customfield_move_field',
            id=id,
            categoryid=categoryid,
            beforeid=beforeid,
        )

    def reload_template(self, component: str, area: str,
                        itemid: int) -> ReloadTemplate:
        """Reloads template

        Args:
            component (str): component
            area (str): area
            itemid (int): itemid

        Returns:
            ReloadTemplate: Response
        """
        data = self.moodle.post(
            'core_customfield_reload_template',
            component=component,
            area=area,
            itemid=itemid,
        )
        return self._tr(ReloadTemplate, **data)



from datetime import datetime
from typing import List, Optional, Union

from moodle import BaseMoodle, MoodleWarning
from . import ContactRequest


class BaseMessage(BaseMoodle):
    def block_contacts(self,
                       userids: List[int],
                       userid: int = 0) -> List[MoodleWarning]:
        """** DEPRECATED ** Please do not call this function any more. Block contacts

        Args:
            userids (List[int]): List of user IDs
            userid (int, optional): The id of the user we are blocking the contacts for, 0 for the current user. Defaults to 0.

        Returns:
            List[Warning]: list of warnings
        """
        res = self.moodle.post(
            'core_message_block_contacts',
            userids=userids,
            userid=userid,
        )
        return [self._tr(MoodleWarning, **data) for data in res] if res else []

    def block_user(self, userid: int,
                   blockeduserid: int) -> List[MoodleWarning]:
        """Blocks a user

        Args:
            userid (int): The id of the user who is blocking
            blockeduserid (int): The id of the user being blocked

        Returns:
            List[Warning]: list of warnings
        """
        res = self.moodle.post(
            'core_message_block_user',
            userid=userid,
            blockeduserid=blockeduserid,
        )
        return [self._tr(MoodleWarning, **data) for data in res] if res else []

    def confirm_contact_request(self, userid: int,
                                requesteduserid: int) -> List[MoodleWarning]:
        """Confirms a contact request

        Args:
            userid (int): The id of the user making the request
            requesteduserid (int): The id of the user being requested

        Returns:
            List[Warning]: list of warnings
        """
        res = self.moodle.post(
            'core_message_confirm_contact_request',
            userid=userid,
            requesteduserid=requesteduserid,
        )
        return [self._tr(MoodleWarning, **data) for data in res] if res else []

    def create_contact_request(self, userid: int,
                               requesteduserid: int) -> ContactRequest:
        """Creates a contact request

        Args:
            userid (int): The id of the user making the request
            requesteduserid (int): The id of the user being requested

        Returns:
            ContactRequest: ContactRequest object
        """
        res = self.moodle.post(
            'core_message_create_contact_request',
            userid=userid,
            requesteduserid=requesteduserid,
        )
        return self._tr(ContactRequest, **res)

    def create_contacts(self,
                        userids: List[int],
                        userid: int = 0) -> List[MoodleWarning]:
        """** DEPRECATED ** Please do not call this function any more. Add contacts to the contact list

        Args:
            userids (List[int]): List of user IDs
            userid (int, optional): The id of the user we are creating the contacts for, 0 for the current user. Defaults to 0.

        Returns:
            List[Warning]: list of warnings
        """
        res = self.moodle.post(
            'core_message_create_contacts',
            userids=userids,
            userid=userid,
        )
        return [self._tr(MoodleWarning, **data) for data in res] if res else []

    def data_for_messagearea_contacts(self,
                                      userid: int,
                                      limitfrom: int = 0,
                                      limitnum: int = 0):
        res = self.moodle.post(
            'core_message_data_for_messagearea_contacts',
            userid=userid,
            limitfrom=limitfrom,
            limitnum=limitnum,
        )
        return res

    def data_for_messagearea_conversations(self,
                                           userid: int,
                                           limitfrom: int = 0,
                                           limitnum: int = 0):
        res = self.moodle.post(
            'core_message_data_for_messagearea_conversations',
            userid=userid,
            limitfrom=limitfrom,
            limitnum=limitnum,
        )
        return res

    def data_for_messagearea_messages(self,
                                      currentuserid: int,
                                      otheruserid: int,
                                      limitfrom: int = 0,
                                      limitnum: int = 0,
                                      newest: Optional[int] = None,
                                      timefrom: Union[datetime, int] = 0):
        res = self.moodle.post(
            'core_message_data_for_messagearea_messages',
            currentuserid=currentuserid,
            otheruserid=otheruserid,
            limitfrom=limitfrom,
            limitnum=limitnum,
            newest=newest,
            timefrom=timefrom,
        )
        return res

    def data_for_messagearea_search_messages(self,
                                             userid: int,
                                             search: str,
                                             limitfrom: int = 0,
                                             limitnum: int = 0):
        res = self.moodle.post(
            'core_message_data_for_messagearea_search_messages',
            userid=userid,
            search=search,
            limitfrom=limitfrom,
            limitnum=limitnum,
        )
        return res

    def decline_contact_request(self, userid: int, requesteduserid: int):
        res = self.moodle.post(
            'core_message_decline_contact_request',
            userid=userid,
            requesteduserid=requesteduserid,
        )
        return res

    def delete_contacts(self, userids: List[int], userid: int = 0):
        res = self.moodle.post(
            'core_message_delete_contacts',
            userids=userids,
            userid=userid,
        )
        return res

    def delete_conversation(self, userid: int, otheruserid: int):
        res = self.moodle.post(
            'core_message_delete_conversation',
            userid=userid,
            otheruserid=otheruserid,
        )
        return res

    def delete_conversations_by_id(self, userid: int,
                                   conversationids: List[int]):
        res = self.moodle.post(
            'core_message_delete_conversations_by_id',
            conversationids=conversationids,
        )
        return res

    def delete_message(self, messageid: int, userid: int, read: int = 1):
        res = self.moodle.post(
            'core_message_delete_message',
            messageid=messageid,
            userid=userid,
            read=read,
        )
        return res

    def delete_message_for_all_users(self, messageid: int, userid: int):
        res = self.moodle.post(
            'core_message_delete_message_for_all_users',
            messageid=messageid,
            userid=userid,
        )
        return res

    def get_blocked_users(self, userid: int):
        res = self.moodle.post('core_message_get_blocked_users', userid=userid)
        return res

    def get_contact_requests(self,
                             userid: int,
                             limitfrom: int = 0,
                             limitnum: int = 0):
        res = self.moodle.post(
            'core_message_get_contact_requests',
            userid=userid,
            limitfrom=limitfrom,
            limitnum=limitnum,
        )
        return res

    def get_contacts(self):
        res = self.moodle.post('core_message_get_contacts')
        return res

    def get_conversation(self,
                         userid: int,
                         conversationid: int,
                         includecontactrequests: int,
                         includeprivacyinfo: int,
                         memberlimit: int = 0,
                         memberoffset: int = 0,
                         messagelimit: int = 100,
                         messageoffset: int = 0,
                         newestmessagesfirst: int = 1):
        res = self.moodle.post(
            'core_message_get_conversation',
            userid=userid,
            conversationid=conversationid,
            includecontactrequests=includecontactrequests,
            includeprivacyinfo=includeprivacyinfo,
            memberlimit=memberlimit,
            memberoffset=memberoffset,
            messagelimit=messagelimit,
            messageoffset=messageoffset,
            newestmessagesfirst=newestmessagesfirst,
        )
        return res

    def get_conversation_between_users(self,
                                       userid: int,
                                       otheruserid: int,
                                       includecontactrequests: int,
                                       includeprivacyinfo: int,
                                       intmemberlimit: int = 0,
                                       memberoffset: int = 0,
                                       messagelimit: int = 100,
                                       messageoffset: int = 0,
                                       newestmessagesfirst: int = 1):
        res = self.moodle.post(
            'core_message_get_conversation_between_users',
            userid=userid,
            otheruserid=otheruserid,
            includecontactrequests=includecontactrequests,
            includeprivacyinfo=includeprivacyinfo,
            intmemberlimit=intmemberlimit,
            memberoffset=memberoffset,
            messagelimit=messagelimit,
            messageoffset=messageoffset,
            newestmessagesfirst=newestmessagesfirst,
        )
        return res

    def get_conversation_counts(self, userid: int = 0):
        res = self.moodle.post('core_message_get_conversation_counts',
                               userid=userid)
        return res

    def get_conversation_members(self,
                                 userid: int,
                                 conversationid: int,
                                 includecontactrequests: int,
                                 includeprivacyinfo: Optional[int] = None,
                                 limitfrom: int = 0,
                                 limitnum: int = 0):
        res = self.moodle.post(
            'core_message_get_conversation_members',
            userid=userid,
            conversationid=conversationid,
            includecontactrequests=includecontactrequests,
            includeprivacyinfo=includeprivacyinfo or '',
            limitfrom=limitfrom,
            limitnum=limitnum,
        )
        return res

    def get_conversation_messages(self,
                                  currentuserid: int,
                                  convid: int,
                                  limitfrom: int = 0,
                                  limitnum: int = 0,
                                  newest: Optional[int] = None,
                                  timefrom: Union[datetime, int] = 0):
        res = self.moodle.post(
            'core_message_get_conversation_messages',
            currentuserid=currentuserid,
            convid=convid,
            limitfrom=limitfrom,
            limitnum=limitnum,
            newest=newest or '',
            timefrom=timefrom,
        )
        return res

    def get_conversations(self,
                          userid: int,
                          limitfrom: int = 0,
                          limitnum: int = 0,
                          type_: Optional[int] = None):
        res = self.moodle.post(
            'core_message_get_conversations',
            userid=userid,
            limitfrom=limitfrom,
            limitnum=limitnum,
            type=type_,
        )
        return res

    def get_member_info(self,
                        referenceuserid: int,
                        userids: List[int],
                        includecontactrequests: Optional[int] = None,
                        includeprivacyinfo: Optional[int] = None):
        res = self.moodle.post(
            'core_message_get_member_info',
            referenceuserid=referenceuserid,
            userids=userids,
            includecontactrequests=includecontactrequests or '',
            includeprivacyinfo=includeprivacyinfo or '',
        )
        return res

    def get_messages(self,
                     useridto: int,
                     useridfrom: int = 0,
                     type_: str = 'both',
                     read: int = 1,
                     newestfirst: int = 1,
                     limitfrom: int = 0,
                     limitnum: int = 0):
        res = self.moodle.post(
            'core_message_get_messages',
            useridto=useridto,
            useridfrom=useridfrom,
            type=type_,
            read=read,
            newestfirst=newestfirst,
            limitfrom=limitfrom,
            limitnum=limitnum,
        )
        return res

    def get_received_contact_requests_count(self, userid: int):
        res = self.moodle.post(
            'core_message_get_received_contact_requests_count', userid=userid)
        return res

    def get_self_conversation(self,
                              userid: int,
                              messagelimit: int = 100,
                              messageoffset: int = 0,
                              newestmessagesfirst: int = 1):
        res = self.moodle.post(
            'core_message_get_self_conversation',
            userid=userid,
            messagelimit=messagelimit,
            messageoffset=messageoffset,
            newestmessagesfirst=newestmessagesfirst,
        )
        return res

    def get_unread_conversation_counts(self, userid: int = 0):
        res = self.moodle.post(
            'core_message_get_unread_conversation_counts',
            userid=userid,
        )
        return res

    def get_unread_conversations_count(self, useridto: int):
        res = self.moodle.post(
            'core_message_get_unread_conversations_count',
            useridto=useridto,
        )
        return res

    def get_user_contacts(self,
                          userid: int,
                          limitfrom: int = 0,
                          limitnum: int = 0):
        res = self.moodle.post(
            'core_message_get_user_contacts',
            userid=userid,
            limitfrom=limitfrom,
            limitnum=limitnum,
        )
        return res

    def get_user_message_preferences(self, userid: int = 0):
        res = self.moodle.post(
            'core_message_get_user_message_preferences',
            userid=userid,
        )
        return res

    def get_user_notification_preferences(self, userid: int = 0):
        res = self.moodle.post(
            'core_message_get_user_notification_preferences',
            userid=userid,
        )
        return res

    def mark_all_conversation_messages_as_read(self, userid: int,
                                               conversationid: int):
        res = self.moodle.post(
            'core_message_mark_all_conversation_messages_as_read',
            userid=userid,
            conversationid=conversationid,
        )
        return res

    def mark_all_messages_as_read(self, useridto: int, useridfrom: int = 0):
        res = self.moodle.post(
            'core_message_mark_all_messages_as_read',
            useridto=useridto,
            useridfrom=useridfrom,
        )
        return res

    def mark_all_notifications_as_read(self,
                                       useridto: int,
                                       useridfrom: int = 0):
        res = self.moodle.post(
            'core_message_mark_all_notifications_as_read',
            useridto=useridto,
            useridfrom=useridfrom,
        )
        return res

    def mark_message_read(self,
                          messageid: int,
                          timeread: Union[datetime, int] = 0):
        res = self.moodle.post(
            'core_message_mark_message_read',
            messageid=messageid,
            timeread=timeread,
        )
        return res

    def mark_notification_read(self,
                               notificationid: int,
                               timeread: Union[datetime, int] = 0):
        res = self.moodle.post(
            'core_message_mark_notification_read',
            notificationid=notificationid,
            timeread=timeread,
        )
        return res

    def message_processor_config_form(self, userid: int, name: str,
                                      formvalues: List[dict]):
        res = self.moodle.post(
            'core_message_message_processor_config_form',
            userid=userid,
            name=name,
            formvalues=formvalues,
        )
        return res

    def message_search_users(self,
                             userid: int,
                             search: str,
                             limitfrom: int = 0,
                             limitnum: int = 0):
        res = self.moodle.post(
            'core_message_message_search_users',
            userid=userid,
            search=search,
            limitfrom=limitfrom,
            limitnum=limitnum,
        )
        return res

    def mute_conversations(self, userid: int, conversationids: List[int]):
        res = self.moodle.post(
            'core_message_mute_conversations',
            userid=userid,
            conversationids=conversationids,
        )
        return res

    def search_contacts(self,
                        searchtext: str,
                        onlymycourses: Optional[int] = None):
        res = self.moodle.post(
            'core_message_search_contacts',
            searchtext=searchtext,
            onlymycourses=onlymycourses,
        )
        return res

    def send_instant_messages(self, messages: List[dict]):
        res = self.moodle.post(
            'core_message_send_instant_messages',
            messages=messages,
        )
        return res

    def send_messages_to_conversation(self, conversationid: int,
                                      messages: List[dict]):
        res = self.moodle.post(
            'core_message_send_messages_to_conversation',
            conversationid=conversationid,
            messages=messages,
        )
        return res

    def set_favourite_conversations(self, userid: int,
                                    conversations: List[int]):
        res = self.moodle.post(
            'core_message_set_favourite_conversations',
            userid=userid,
            conversations=conversations,
        )
        return res

    def unblock_contacts(self, userids: List[int], userid: int = 0):
        res = self.moodle.post(
            'core_message_unblock_contacts',
            userids=userids,
            userid=userid,
        )
        return res

    def unblock_user(self, userid: int, conversationids: List[int]):
        res = self.moodle.post(
            'core_message_unblock_user',
            userid=userid,
            conversationids=conversationids,
        )
        return res

    def unmute_conversations(self, userid: int, conversationids: List[int]):
        res = self.moodle.post(
            'core_message_unmute_conversations',
            userid=userid,
            conversationids=conversationids,
        )
        return res

    def unset_favourite_conversations(self,
                                      userid: int = 0,
                                      conversations: List[int] = None):
        res = self.moodle.post(
            'core_message_unset_favourite_conversations',
            userid=userid,
            conversations=conversations or [0],
        )
        return res



from moodle.attr import dataclass
from moodle import MoodleWarning
from typing import List


@dataclass
class SignupUserResponse:
    """SignupUser
    Args:
        success (int): True if the user was created false otherwise
        warnings (List[Warning]): list of warnings
    """
    success: int
    warnings: List[MoodleWarning]



from moodle.attr import dataclass


@dataclass
class BackupCompletionStatus:
    """Backup completion status

    Args:
        status (int): Backup Status
        progress (float): Backup progress
        backupid (str): Backup id
        operation (str): operation type
    """
    status: int
    progress: float
    backupid: str
    operation: str



from moodle.attr import dataclass


@dataclass
class GeneralNameValue:
    """General NameValue
    Args:
        name (str): Name
        value (str): Value
    """
    name: str
    value: str

    def __str__(self) -> str:
        return f"{self.name} : {self.value}"



from typing import List

from moodle import BaseMoodle
from moodle.base.general import GeneralStatus
from .page import PagesResponse


class BasePage(BaseMoodle):
    def get_pages_by_courses(self, courseids: List[int]) -> PagesResponse:
        """Returns a list of pages in a provided list of courses, if no list is provided all pages that the user can view will be returned.

        Args:
            courseids (List[int]): Array of course ids

        Returns:
            PagesResponse: Response
        """
        data = self.moodle.post(
            'mod_page_get_pages_by_courses',
            courseids=courseids,
        )
        return self._tr(PagesResponse, **data)

    def view_page(self, pageid: int) -> GeneralStatus:
        """Simulate the view.php web interface page: trigger events, completion, etc...

        Args:
            pageid (int): page instance id

        Returns:
            GeneralStatus: Response
        """
        data = self.moodle.post(
            'mod_page_view_page',
            pageid=pageid,
        )
        return self._tr(GeneralStatus, **data)
